from typing import Any, Dict, List, Optional, Tuple, Union
from graphql import DocumentNode, NamedTypeNode, NameNode, OperationDefinitionNode, OperationType, SelectionNode, SelectionSetNode, VariableDefinitionNode, VariableNode, print_ast
from .add_external_model import AddExternalModel
from .add_hf_model import AddHFModel
from .add_model import AddModel
from .add_remote_env import AddRemoteEnv
from .attach_model_to_use_case import AttachModelToUseCase
from .base_client_open_telemetry import BaseClientOpenTelemetry
from .base_model import UNSET, UnsetType, Upload
from .base_operation import GraphQLField
from .cancel_ab_campaign import CancelABCampaign
from .cancel_job import CancelJob
from .create_ab_campaign import CreateAbCampaign
from .create_custom_recipe import CreateCustomRecipe
from .create_grader import CreateGrader
from .create_job import CreateJob
from .create_judge import CreateJudge
from .create_metric import CreateMetric
from .create_prebuilt_judge import CreatePrebuiltJudge
from .create_role import CreateRole
from .create_team import CreateTeam
from .create_use_case import CreateUseCase
from .create_user import CreateUser
from .delete_custom_recipe import DeleteCustomRecipe
from .delete_dataset import DeleteDataset
from .delete_grader import DeleteGrader
from .delete_judge import DeleteJudge
from .delete_user import DeleteUser
from .deploy_model import DeployModel
from .describe_ab_campaign import DescribeAbCampaign
from .describe_dataset import DescribeDataset
from .describe_interaction import DescribeInteraction
from .describe_job import DescribeJob
from .describe_metric import DescribeMetric
from .describe_metric_admin import DescribeMetricAdmin
from .describe_model import DescribeModel
from .describe_model_admin import DescribeModelAdmin
from .describe_use_case import DescribeUseCase
from .enums import CompletionGroupBy
from .get_custom_recipe import GetCustomRecipe
from .get_grader import GetGrader
from .get_judge import GetJudge
from .input_types import AbcampaignCreate, AbCampaignFilter, AddExternalModelInput, AddHFModelInput, AddModelInput, AttachModel, CreateRecipeInput, CursorPageInput, CustomRecipeFilterInput, DatasetCreate, GraderCreateInput, GraderUpdateInput, JobInput, JudgeCreate, JudgeUpdate, ListCompletionsFilterInput, ListJobsFilterInput, MetricCreate, MetricLink, MetricUnlink, ModelComputeConfigInput, ModelFilter, ModelPlacementInput, OrderPair, PrebuiltJudgeCreate, RemoteEnvCreate, RoleCreate, TeamCreate, TeamMemberRemove, TeamMemberSet, UpdateModelService, UpdateRecipeInput, UseCaseCreate, UseCaseShares, UserCreate
from .link_metric import LinkMetric
from .list_ab_campaigns import ListAbCampaigns
from .list_compute_pools import ListComputePools
from .list_custom_recipes import ListCustomRecipes
from .list_datasets import ListDatasets
from .list_graders import ListGraders
from .list_grouped_interactions import ListGroupedInteractions
from .list_harmony_groups import ListHarmonyGroups
from .list_interactions import ListInteractions
from .list_jobs import ListJobs
from .list_judge_versions import ListJudgeVersions
from .list_judges import ListJudges
from .list_metrics import ListMetrics
from .list_models import ListModels
from .list_permissions import ListPermissions
from .list_remote_envs import ListRemoteEnvs
from .list_roles import ListRoles
from .list_teams import ListTeams
from .list_use_cases import ListUseCases
from .list_users import ListUsers
from .load_dataset import LoadDataset
from .lock_grader import LockGrader
from .me import Me
from .remove_remote_env import RemoveRemoteEnv
from .remove_team_member import RemoveTeamMember
from .share_use_case import ShareUseCase
from .terminate_model import TerminateModel
from .test_remote_env import TestRemoteEnv
from .unlink_metric import UnlinkMetric
from .update_custom_recipe import UpdateCustomRecipe
from .update_grader import UpdateGrader
from .update_judge import UpdateJudge
from .update_model import UpdateModel
from .update_model_compute_config import UpdateModelComputeConfig
from .update_user import UpdateUser

def gql(q: str) -> str:
    return q

class GQLClient(BaseClientOpenTelemetry):
    """@private"""

    def create_metric(self, input: MetricCreate, **kwargs: Any) -> CreateMetric:
        query = gql('\n            mutation CreateMetric($input: MetricCreate!) {\n              createMetric(input: $input) {\n                ...MetricData\n              }\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='CreateMetric', variables=variables, **kwargs)
        data = self.get_data(response)
        return CreateMetric.model_validate(data)

    def link_metric(self, input: MetricLink, **kwargs: Any) -> LinkMetric:
        query = gql('\n            mutation LinkMetric($input: MetricLink!) {\n              linkMetric(input: $input) {\n                ...MetricWithContextData\n              }\n            }\n\n            fragment MetricWithContextData on MetricWithContext {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='LinkMetric', variables=variables, **kwargs)
        data = self.get_data(response)
        return LinkMetric.model_validate(data)

    def unlink_metric(self, input: MetricUnlink, **kwargs: Any) -> UnlinkMetric:
        query = gql('\n            mutation UnlinkMetric($input: MetricUnlink!) {\n              unlinkMetric(input: $input)\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='UnlinkMetric', variables=variables, **kwargs)
        data = self.get_data(response)
        return UnlinkMetric.model_validate(data)

    def attach_model_to_use_case(self, input: AttachModel, **kwargs: Any) -> AttachModelToUseCase:
        query = gql('\n            mutation AttachModelToUseCase($input: AttachModel!) {\n              attachModel(input: $input) {\n                ...ModelServiceData\n              }\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n\n            fragment ModelServiceData on ModelService {\n              id\n              key\n              name\n              model {\n                ...ModelData\n                backbone {\n                  ...ModelData\n                }\n              }\n              attached\n              isDefault\n              desiredOnline\n              createdAt\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='AttachModelToUseCase', variables=variables, **kwargs)
        data = self.get_data(response)
        return AttachModelToUseCase.model_validate(data)

    def add_external_model(self, input: AddExternalModelInput, **kwargs: Any) -> AddExternalModel:
        query = gql('\n            mutation AddExternalModel($input: AddExternalModelInput!) {\n              addExternalModel(input: $input) {\n                ...ModelData\n              }\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='AddExternalModel', variables=variables, **kwargs)
        data = self.get_data(response)
        return AddExternalModel.model_validate(data)

    def add_model(self, input: AddModelInput, **kwargs: Any) -> AddModel:
        query = gql('\n            mutation AddModel($input: AddModelInput!) {\n              addModel(input: $input) {\n                ...ModelData\n                backbone {\n                  ...ModelData\n                }\n              }\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='AddModel', variables=variables, **kwargs)
        data = self.get_data(response)
        return AddModel.model_validate(data)

    def update_model(self, input: UpdateModelService, **kwargs: Any) -> UpdateModel:
        query = gql('\n            mutation UpdateModel($input: UpdateModelService!) {\n              updateModelService(input: $input) {\n                ...ModelServiceData\n              }\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n\n            fragment ModelServiceData on ModelService {\n              id\n              key\n              name\n              model {\n                ...ModelData\n                backbone {\n                  ...ModelData\n                }\n              }\n              attached\n              isDefault\n              desiredOnline\n              createdAt\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='UpdateModel', variables=variables, **kwargs)
        data = self.get_data(response)
        return UpdateModel.model_validate(data)

    def deploy_model(self, id_or_key: str, wait: bool, placement: Union[Optional[ModelPlacementInput], UnsetType]=UNSET, **kwargs: Any) -> DeployModel:
        query = gql('\n            mutation DeployModel($idOrKey: IdOrKey!, $wait: Boolean!, $placement: ModelPlacementInput) {\n              deployModel(idOrKey: $idOrKey, wait: $wait, placement: $placement)\n            }\n            ')
        variables: Dict[str, object] = {'idOrKey': id_or_key, 'wait': wait, 'placement': placement}
        response = self.execute(query=query, operation_name='DeployModel', variables=variables, **kwargs)
        data = self.get_data(response)
        return DeployModel.model_validate(data)

    def terminate_model(self, id_or_key: str, force: bool, **kwargs: Any) -> TerminateModel:
        query = gql('\n            mutation TerminateModel($idOrKey: IdOrKey!, $force: Boolean!) {\n              terminateModel(idOrKey: $idOrKey, force: $force)\n            }\n            ')
        variables: Dict[str, object] = {'idOrKey': id_or_key, 'force': force}
        response = self.execute(query=query, operation_name='TerminateModel', variables=variables, **kwargs)
        data = self.get_data(response)
        return TerminateModel.model_validate(data)

    def create_use_case(self, input: UseCaseCreate, **kwargs: Any) -> CreateUseCase:
        query = gql('\n            mutation CreateUseCase($input: UseCaseCreate!) {\n              createUseCase(input: $input) {\n                ...UseCaseData\n              }\n            }\n\n            fragment MetricWithContextData on MetricWithContext {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n\n            fragment ModelServiceData on ModelService {\n              id\n              key\n              name\n              model {\n                ...ModelData\n                backbone {\n                  ...ModelData\n                }\n              }\n              attached\n              isDefault\n              desiredOnline\n              createdAt\n            }\n\n            fragment UseCaseData on UseCase {\n              id\n              key\n              name\n              description\n              createdAt\n              metrics {\n                ...MetricWithContextData\n              }\n              modelServices {\n                ...ModelServiceData\n              }\n              permissions\n              shares {\n                team {\n                  id\n                  key\n                  name\n                  createdAt\n                }\n                role {\n                  id\n                  key\n                  name\n                  createdAt\n                  permissions\n                }\n                isOwner\n              }\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='CreateUseCase', variables=variables, **kwargs)
        data = self.get_data(response)
        return CreateUseCase.model_validate(data)

    def share_use_case(self, id_or_key: str, input: UseCaseShares, **kwargs: Any) -> ShareUseCase:
        query = gql('\n            mutation ShareUseCase($idOrKey: IdOrKey!, $input: UseCaseShares!) {\n              shareUseCase(idOrKey: $idOrKey, input: $input) {\n                ...UseCaseData\n              }\n            }\n\n            fragment MetricWithContextData on MetricWithContext {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n\n            fragment ModelServiceData on ModelService {\n              id\n              key\n              name\n              model {\n                ...ModelData\n                backbone {\n                  ...ModelData\n                }\n              }\n              attached\n              isDefault\n              desiredOnline\n              createdAt\n            }\n\n            fragment UseCaseData on UseCase {\n              id\n              key\n              name\n              description\n              createdAt\n              metrics {\n                ...MetricWithContextData\n              }\n              modelServices {\n                ...ModelServiceData\n              }\n              permissions\n              shares {\n                team {\n                  id\n                  key\n                  name\n                  createdAt\n                }\n                role {\n                  id\n                  key\n                  name\n                  createdAt\n                  permissions\n                }\n                isOwner\n              }\n            }\n            ')
        variables: Dict[str, object] = {'idOrKey': id_or_key, 'input': input}
        response = self.execute(query=query, operation_name='ShareUseCase', variables=variables, **kwargs)
        data = self.get_data(response)
        return ShareUseCase.model_validate(data)

    def create_ab_campaign(self, input: AbcampaignCreate, **kwargs: Any) -> CreateAbCampaign:
        query = gql('\n            mutation CreateAbCampaign($input: AbcampaignCreate!) {\n              createAbCampaign(input: $input) {\n                ...AbCampaignCreateData\n              }\n            }\n\n            fragment AbCampaignCreateData on Abcampaign {\n              id\n              key\n              status\n              beginDate\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='CreateAbCampaign', variables=variables, **kwargs)
        data = self.get_data(response)
        return CreateAbCampaign.model_validate(data)

    def cancel_ab_campaign(self, input: str, **kwargs: Any) -> CancelABCampaign:
        query = gql('\n            mutation CancelABCampaign($input: IdOrKey!) {\n              cancelAbCampaign(input: $input)\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='CancelABCampaign', variables=variables, **kwargs)
        data = self.get_data(response)
        return CancelABCampaign.model_validate(data)

    def load_dataset(self, input: DatasetCreate, file: Upload, **kwargs: Any) -> LoadDataset:
        query = gql('\n            mutation LoadDataset($input: DatasetCreate!, $file: Upload!) {\n              createDataset(input: $input, file: $file) {\n                ...DatasetData\n              }\n            }\n\n            fragment DatasetData on Dataset {\n              id\n              key\n              name\n              createdAt\n              kind\n              records\n              metricsUsage {\n                feedbackCount\n                comparisonCount\n                metric {\n                  ...MetricData\n                }\n              }\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n            ')
        variables: Dict[str, object] = {'input': input, 'file': file}
        response = self.execute(query=query, operation_name='LoadDataset', variables=variables, **kwargs)
        data = self.get_data(response)
        return LoadDataset.model_validate(data)

    def delete_dataset(self, id_or_key: str, use_case: str, **kwargs: Any) -> DeleteDataset:
        query = gql('\n            mutation DeleteDataset($idOrKey: IdOrKey!, $useCase: IdOrKey!) {\n              deleteDataset(idOrKey: $idOrKey, useCase: $useCase)\n            }\n            ')
        variables: Dict[str, object] = {'idOrKey': id_or_key, 'useCase': use_case}
        response = self.execute(query=query, operation_name='DeleteDataset', variables=variables, **kwargs)
        data = self.get_data(response)
        return DeleteDataset.model_validate(data)

    def add_hf_model(self, input: AddHFModelInput, **kwargs: Any) -> AddHFModel:
        query = gql('\n            mutation AddHFModel($input: AddHFModelInput!) {\n              importHfModel(input: $input) {\n                ...JobData\n              }\n            }\n\n            fragment CustomRecipeData on CustomRecipe {\n              id\n              key\n              name\n              content\n              contentHash\n              editable\n              global\n              builtin\n              inputSchema\n              jsonSchema\n              description\n              labels {\n                key\n                value\n              }\n              createdAt\n              updatedAt\n              createdBy {\n                id\n                name\n                email\n              }\n            }\n\n            fragment JobData on Job {\n              id\n              name\n              status\n              createdAt\n              createdBy {\n                id\n                name\n              }\n              startedAt\n              endedAt\n              durationMs\n              progress\n              error\n              kind\n              stages {\n                name\n                status\n                info {\n                  __typename\n                  ... on TrainingJobStageOutput {\n                    monitoringLink\n                    totalNumSamples\n                    processedNumSamples\n                    checkpoints\n                  }\n                  ... on EvalJobStageOutput {\n                    totalNumSamples\n                    processedNumSamples\n                  }\n                  ... on BatchInferenceJobStageOutput {\n                    totalNumSamples\n                    processedNumSamples\n                  }\n                }\n              }\n              useCase {\n                id\n                key\n                name\n              }\n              recipe {\n                ...CustomRecipeData\n              }\n              details {\n                args\n                recipeHash\n                artifacts {\n                  id\n                  name\n                  kind\n                  uri\n                  metadata\n                  createdAt\n                }\n              }\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='AddHFModel', variables=variables, **kwargs)
        data = self.get_data(response)
        return AddHFModel.model_validate(data)

    def create_user(self, input: UserCreate, **kwargs: Any) -> CreateUser:
        query = gql('\n            mutation CreateUser($input: UserCreate!) {\n              createUser(input: $input) {\n                ...UserData\n              }\n            }\n\n            fragment UserData on User {\n              id\n              email\n              name\n              createdAt\n              teams {\n                team {\n                  id\n                  key\n                  name\n                  createdAt\n                }\n                role {\n                  id\n                  key\n                  name\n                  createdAt\n                  permissions\n                }\n              }\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='CreateUser', variables=variables, **kwargs)
        data = self.get_data(response)
        return CreateUser.model_validate(data)

    def delete_user(self, user: str, **kwargs: Any) -> DeleteUser:
        query = gql('\n            mutation DeleteUser($user: IdOrKey!) {\n              deleteUser(user: $user) {\n                ...UserData\n              }\n            }\n\n            fragment UserData on User {\n              id\n              email\n              name\n              createdAt\n              teams {\n                team {\n                  id\n                  key\n                  name\n                  createdAt\n                }\n                role {\n                  id\n                  key\n                  name\n                  createdAt\n                  permissions\n                }\n              }\n            }\n            ')
        variables: Dict[str, object] = {'user': user}
        response = self.execute(query=query, operation_name='DeleteUser', variables=variables, **kwargs)
        data = self.get_data(response)
        return DeleteUser.model_validate(data)

    def update_user(self, input: TeamMemberSet, **kwargs: Any) -> UpdateUser:
        query = gql('\n            mutation UpdateUser($input: TeamMemberSet!) {\n              setTeamMember(input: $input) {\n                user {\n                  ...UserData\n                }\n                team {\n                  id\n                  key\n                  name\n                  createdAt\n                }\n                role {\n                  id\n                  key\n                  name\n                  createdAt\n                  permissions\n                }\n              }\n            }\n\n            fragment UserData on User {\n              id\n              email\n              name\n              createdAt\n              teams {\n                team {\n                  id\n                  key\n                  name\n                  createdAt\n                }\n                role {\n                  id\n                  key\n                  name\n                  createdAt\n                  permissions\n                }\n              }\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='UpdateUser', variables=variables, **kwargs)
        data = self.get_data(response)
        return UpdateUser.model_validate(data)

    def create_role(self, input: RoleCreate, **kwargs: Any) -> CreateRole:
        query = gql('\n            mutation CreateRole($input: RoleCreate!) {\n              createRole(input: $input) {\n                id\n                key\n                name\n                createdAt\n                permissions\n              }\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='CreateRole', variables=variables, **kwargs)
        data = self.get_data(response)
        return CreateRole.model_validate(data)

    def create_team(self, input: TeamCreate, **kwargs: Any) -> CreateTeam:
        query = gql('\n            mutation CreateTeam($input: TeamCreate!) {\n              createTeam(input: $input) {\n                id\n                key\n                name\n                createdAt\n              }\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='CreateTeam', variables=variables, **kwargs)
        data = self.get_data(response)
        return CreateTeam.model_validate(data)

    def update_model_compute_config(self, id_or_key: str, input: ModelComputeConfigInput, **kwargs: Any) -> UpdateModelComputeConfig:
        query = gql('\n            mutation UpdateModelComputeConfig($idOrKey: IdOrKey!, $input: ModelComputeConfigInput!) {\n              updateModelComputeConfig(idOrKey: $idOrKey, input: $input) {\n                ...ModelData\n              }\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n            ')
        variables: Dict[str, object] = {'idOrKey': id_or_key, 'input': input}
        response = self.execute(query=query, operation_name='UpdateModelComputeConfig', variables=variables, **kwargs)
        data = self.get_data(response)
        return UpdateModelComputeConfig.model_validate(data)

    def add_remote_env(self, input: RemoteEnvCreate, **kwargs: Any) -> AddRemoteEnv:
        query = gql('\n            mutation AddRemoteEnv($input: RemoteEnvCreate!) {\n              addRemoteEnv(input: $input) {\n                ...RemoteEnvData\n              }\n            }\n\n            fragment RemoteEnvData on RemoteEnv {\n              id\n              key\n              name\n              url\n              description\n              createdAt\n              version\n              status\n              metadataSchema\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='AddRemoteEnv', variables=variables, **kwargs)
        data = self.get_data(response)
        return AddRemoteEnv.model_validate(data)

    def remove_remote_env(self, id_or_key: str, **kwargs: Any) -> RemoveRemoteEnv:
        query = gql('\n            mutation RemoveRemoteEnv($idOrKey: IdOrKey!) {\n              removeRemoteEnv(idOrKey: $idOrKey)\n            }\n            ')
        variables: Dict[str, object] = {'idOrKey': id_or_key}
        response = self.execute(query=query, operation_name='RemoveRemoteEnv', variables=variables, **kwargs)
        data = self.get_data(response)
        return RemoveRemoteEnv.model_validate(data)

    def test_remote_env(self, input: RemoteEnvCreate, **kwargs: Any) -> TestRemoteEnv:
        query = gql('\n            mutation TestRemoteEnv($input: RemoteEnvCreate!) {\n              testRemoteEnv(input: $input) {\n                __typename\n                ... on RemoteEnvTestOffline {\n                  error\n                }\n                ... on RemoteEnvTestOnline {\n                  name\n                  version\n                  description\n                }\n              }\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='TestRemoteEnv', variables=variables, **kwargs)
        data = self.get_data(response)
        return TestRemoteEnv.model_validate(data)

    def create_custom_recipe(self, use_case: str, input: CreateRecipeInput, file: Upload, **kwargs: Any) -> CreateCustomRecipe:
        query = gql('\n            mutation CreateCustomRecipe($useCase: IdOrKey!, $input: CreateRecipeInput!, $file: Upload!) {\n              createCustomRecipe(useCase: $useCase, input: $input, file: $file) {\n                ...CustomRecipeData\n              }\n            }\n\n            fragment CustomRecipeData on CustomRecipe {\n              id\n              key\n              name\n              content\n              contentHash\n              editable\n              global\n              builtin\n              inputSchema\n              jsonSchema\n              description\n              labels {\n                key\n                value\n              }\n              createdAt\n              updatedAt\n              createdBy {\n                id\n                name\n                email\n              }\n            }\n            ')
        variables: Dict[str, object] = {'useCase': use_case, 'input': input, 'file': file}
        response = self.execute(query=query, operation_name='CreateCustomRecipe', variables=variables, **kwargs)
        data = self.get_data(response)
        return CreateCustomRecipe.model_validate(data)

    def update_custom_recipe(self, use_case: str, id: str, input: UpdateRecipeInput, file: Union[Optional[Upload], UnsetType]=UNSET, **kwargs: Any) -> UpdateCustomRecipe:
        query = gql('\n            mutation UpdateCustomRecipe($useCase: IdOrKey!, $id: IdOrKey!, $input: UpdateRecipeInput!, $file: Upload) {\n              updateCustomRecipe(useCase: $useCase, id: $id, input: $input, file: $file) {\n                ...CustomRecipeData\n              }\n            }\n\n            fragment CustomRecipeData on CustomRecipe {\n              id\n              key\n              name\n              content\n              contentHash\n              editable\n              global\n              builtin\n              inputSchema\n              jsonSchema\n              description\n              labels {\n                key\n                value\n              }\n              createdAt\n              updatedAt\n              createdBy {\n                id\n                name\n                email\n              }\n            }\n            ')
        variables: Dict[str, object] = {'useCase': use_case, 'id': id, 'input': input, 'file': file}
        response = self.execute(query=query, operation_name='UpdateCustomRecipe', variables=variables, **kwargs)
        data = self.get_data(response)
        return UpdateCustomRecipe.model_validate(data)

    def delete_custom_recipe(self, use_case: str, id: str, **kwargs: Any) -> DeleteCustomRecipe:
        query = gql('\n            mutation DeleteCustomRecipe($useCase: IdOrKey!, $id: IdOrKey!) {\n              deleteCustomRecipe(useCase: $useCase, id: $id)\n            }\n            ')
        variables: Dict[str, object] = {'useCase': use_case, 'id': id}
        response = self.execute(query=query, operation_name='DeleteCustomRecipe', variables=variables, **kwargs)
        data = self.get_data(response)
        return DeleteCustomRecipe.model_validate(data)

    def create_judge(self, use_case: str, input: JudgeCreate, **kwargs: Any) -> CreateJudge:
        query = gql('\n            mutation CreateJudge($useCase: IdOrKey!, $input: JudgeCreate!) {\n              createJudge(useCase: $useCase, input: $input) {\n                ...JudgeData\n              }\n            }\n\n            fragment JudgeData on Judge {\n              id\n              key\n              version\n              name\n              criteria\n              prebuilt\n              examples {\n                input {\n                  role\n                  content\n                }\n                output\n                pass\n                reasoning\n              }\n              capabilities\n              model {\n                ...ModelData\n              }\n              useCaseId\n              metric {\n                ...MetricData\n              }\n              createdAt\n              updatedAt\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n            ')
        variables: Dict[str, object] = {'useCase': use_case, 'input': input}
        response = self.execute(query=query, operation_name='CreateJudge', variables=variables, **kwargs)
        data = self.get_data(response)
        return CreateJudge.model_validate(data)

    def create_prebuilt_judge(self, use_case: str, input: PrebuiltJudgeCreate, **kwargs: Any) -> CreatePrebuiltJudge:
        query = gql('\n            mutation CreatePrebuiltJudge($useCase: IdOrKey!, $input: PrebuiltJudgeCreate!) {\n              createPrebuiltJudge(useCase: $useCase, input: $input) {\n                ...JudgeData\n              }\n            }\n\n            fragment JudgeData on Judge {\n              id\n              key\n              version\n              name\n              criteria\n              prebuilt\n              examples {\n                input {\n                  role\n                  content\n                }\n                output\n                pass\n                reasoning\n              }\n              capabilities\n              model {\n                ...ModelData\n              }\n              useCaseId\n              metric {\n                ...MetricData\n              }\n              createdAt\n              updatedAt\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n            ')
        variables: Dict[str, object] = {'useCase': use_case, 'input': input}
        response = self.execute(query=query, operation_name='CreatePrebuiltJudge', variables=variables, **kwargs)
        data = self.get_data(response)
        return CreatePrebuiltJudge.model_validate(data)

    def update_judge(self, use_case: str, key: str, input: JudgeUpdate, **kwargs: Any) -> UpdateJudge:
        query = gql('\n            mutation UpdateJudge($useCase: IdOrKey!, $key: String!, $input: JudgeUpdate!) {\n              updateJudge(useCase: $useCase, key: $key, input: $input) {\n                ...JudgeData\n              }\n            }\n\n            fragment JudgeData on Judge {\n              id\n              key\n              version\n              name\n              criteria\n              prebuilt\n              examples {\n                input {\n                  role\n                  content\n                }\n                output\n                pass\n                reasoning\n              }\n              capabilities\n              model {\n                ...ModelData\n              }\n              useCaseId\n              metric {\n                ...MetricData\n              }\n              createdAt\n              updatedAt\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n            ')
        variables: Dict[str, object] = {'useCase': use_case, 'key': key, 'input': input}
        response = self.execute(query=query, operation_name='UpdateJudge', variables=variables, **kwargs)
        data = self.get_data(response)
        return UpdateJudge.model_validate(data)

    def delete_judge(self, use_case: str, key: str, **kwargs: Any) -> DeleteJudge:
        query = gql('\n            mutation DeleteJudge($useCase: IdOrKey!, $key: String!) {\n              deleteJudge(useCase: $useCase, key: $key) {\n                success\n                details\n              }\n            }\n            ')
        variables: Dict[str, object] = {'useCase': use_case, 'key': key}
        response = self.execute(query=query, operation_name='DeleteJudge', variables=variables, **kwargs)
        data = self.get_data(response)
        return DeleteJudge.model_validate(data)

    def create_grader(self, use_case: str, input: GraderCreateInput, **kwargs: Any) -> CreateGrader:
        query = gql('\n            mutation CreateGrader($useCase: IdOrKey!, $input: GraderCreateInput!) {\n              createGrader(useCase: $useCase, input: $input) {\n                ...GraderData\n              }\n            }\n\n            fragment GraderData on Grader {\n              id\n              name\n              key\n              locked\n              graderType\n              graderConfig {\n                __typename\n                ... on JudgeConfigOutput {\n                  judgeCriteria: criteria\n                  examples {\n                    input {\n                      role\n                      content\n                    }\n                    output\n                    pass\n                    reasoning\n                  }\n                  model {\n                    ...ModelData\n                  }\n                }\n                ... on PrebuiltConfigOutput {\n                  prebuiltCriteria: criteria {\n                    key\n                    name\n                    feedbackKey\n                    description\n                  }\n                  model {\n                    ...ModelData\n                  }\n                }\n                ... on RemoteConfigOutput {\n                  url\n                  version\n                  description\n                }\n                ... on CustomConfigOutput {\n                  graderDescription: description\n                }\n              }\n              useCase {\n                id\n                key\n                name\n              }\n              metric {\n                ...MetricData\n              }\n              createdAt\n              updatedAt\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n            ')
        variables: Dict[str, object] = {'useCase': use_case, 'input': input}
        response = self.execute(query=query, operation_name='CreateGrader', variables=variables, **kwargs)
        data = self.get_data(response)
        return CreateGrader.model_validate(data)

    def update_grader(self, use_case: str, id: str, input: GraderUpdateInput, **kwargs: Any) -> UpdateGrader:
        query = gql('\n            mutation UpdateGrader($useCase: IdOrKey!, $id: IdOrKey!, $input: GraderUpdateInput!) {\n              updateGrader(useCase: $useCase, id: $id, input: $input) {\n                ...GraderData\n              }\n            }\n\n            fragment GraderData on Grader {\n              id\n              name\n              key\n              locked\n              graderType\n              graderConfig {\n                __typename\n                ... on JudgeConfigOutput {\n                  judgeCriteria: criteria\n                  examples {\n                    input {\n                      role\n                      content\n                    }\n                    output\n                    pass\n                    reasoning\n                  }\n                  model {\n                    ...ModelData\n                  }\n                }\n                ... on PrebuiltConfigOutput {\n                  prebuiltCriteria: criteria {\n                    key\n                    name\n                    feedbackKey\n                    description\n                  }\n                  model {\n                    ...ModelData\n                  }\n                }\n                ... on RemoteConfigOutput {\n                  url\n                  version\n                  description\n                }\n                ... on CustomConfigOutput {\n                  graderDescription: description\n                }\n              }\n              useCase {\n                id\n                key\n                name\n              }\n              metric {\n                ...MetricData\n              }\n              createdAt\n              updatedAt\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n            ')
        variables: Dict[str, object] = {'useCase': use_case, 'id': id, 'input': input}
        response = self.execute(query=query, operation_name='UpdateGrader', variables=variables, **kwargs)
        data = self.get_data(response)
        return UpdateGrader.model_validate(data)

    def delete_grader(self, use_case: str, id: str, **kwargs: Any) -> DeleteGrader:
        query = gql('\n            mutation DeleteGrader($useCase: IdOrKey!, $id: IdOrKey!) {\n              deleteGrader(useCase: $useCase, id: $id) {\n                success\n                details\n              }\n            }\n            ')
        variables: Dict[str, object] = {'useCase': use_case, 'id': id}
        response = self.execute(query=query, operation_name='DeleteGrader', variables=variables, **kwargs)
        data = self.get_data(response)
        return DeleteGrader.model_validate(data)

    def lock_grader(self, use_case: str, id: str, locked: bool, **kwargs: Any) -> LockGrader:
        query = gql('\n            mutation LockGrader($useCase: IdOrKey!, $id: IdOrKey!, $locked: Boolean!) {\n              lockGrader(useCase: $useCase, id: $id, locked: $locked) {\n                ...GraderData\n              }\n            }\n\n            fragment GraderData on Grader {\n              id\n              name\n              key\n              locked\n              graderType\n              graderConfig {\n                __typename\n                ... on JudgeConfigOutput {\n                  judgeCriteria: criteria\n                  examples {\n                    input {\n                      role\n                      content\n                    }\n                    output\n                    pass\n                    reasoning\n                  }\n                  model {\n                    ...ModelData\n                  }\n                }\n                ... on PrebuiltConfigOutput {\n                  prebuiltCriteria: criteria {\n                    key\n                    name\n                    feedbackKey\n                    description\n                  }\n                  model {\n                    ...ModelData\n                  }\n                }\n                ... on RemoteConfigOutput {\n                  url\n                  version\n                  description\n                }\n                ... on CustomConfigOutput {\n                  graderDescription: description\n                }\n              }\n              useCase {\n                id\n                key\n                name\n              }\n              metric {\n                ...MetricData\n              }\n              createdAt\n              updatedAt\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n            ')
        variables: Dict[str, object] = {'useCase': use_case, 'id': id, 'locked': locked}
        response = self.execute(query=query, operation_name='LockGrader', variables=variables, **kwargs)
        data = self.get_data(response)
        return LockGrader.model_validate(data)

    def remove_team_member(self, input: TeamMemberRemove, **kwargs: Any) -> RemoveTeamMember:
        query = gql('\n            mutation RemoveTeamMember($input: TeamMemberRemove!) {\n              removeTeamMember(input: $input) {\n                ...UserData\n              }\n            }\n\n            fragment UserData on User {\n              id\n              email\n              name\n              createdAt\n              teams {\n                team {\n                  id\n                  key\n                  name\n                  createdAt\n                }\n                role {\n                  id\n                  key\n                  name\n                  createdAt\n                  permissions\n                }\n              }\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='RemoveTeamMember', variables=variables, **kwargs)
        data = self.get_data(response)
        return RemoveTeamMember.model_validate(data)

    def create_job(self, input: JobInput, **kwargs: Any) -> CreateJob:
        query = gql('\n            mutation CreateJob($input: JobInput!) {\n              createJob(input: $input) {\n                ...JobData\n              }\n            }\n\n            fragment CustomRecipeData on CustomRecipe {\n              id\n              key\n              name\n              content\n              contentHash\n              editable\n              global\n              builtin\n              inputSchema\n              jsonSchema\n              description\n              labels {\n                key\n                value\n              }\n              createdAt\n              updatedAt\n              createdBy {\n                id\n                name\n                email\n              }\n            }\n\n            fragment JobData on Job {\n              id\n              name\n              status\n              createdAt\n              createdBy {\n                id\n                name\n              }\n              startedAt\n              endedAt\n              durationMs\n              progress\n              error\n              kind\n              stages {\n                name\n                status\n                info {\n                  __typename\n                  ... on TrainingJobStageOutput {\n                    monitoringLink\n                    totalNumSamples\n                    processedNumSamples\n                    checkpoints\n                  }\n                  ... on EvalJobStageOutput {\n                    totalNumSamples\n                    processedNumSamples\n                  }\n                  ... on BatchInferenceJobStageOutput {\n                    totalNumSamples\n                    processedNumSamples\n                  }\n                }\n              }\n              useCase {\n                id\n                key\n                name\n              }\n              recipe {\n                ...CustomRecipeData\n              }\n              details {\n                args\n                recipeHash\n                artifacts {\n                  id\n                  name\n                  kind\n                  uri\n                  metadata\n                  createdAt\n                }\n              }\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='CreateJob', variables=variables, **kwargs)
        data = self.get_data(response)
        return CreateJob.model_validate(data)

    def cancel_job(self, job_id: Any, **kwargs: Any) -> CancelJob:
        query = gql('\n            mutation CancelJob($jobId: UUID!) {\n              cancelJob(id: $jobId) {\n                ...JobData\n              }\n            }\n\n            fragment CustomRecipeData on CustomRecipe {\n              id\n              key\n              name\n              content\n              contentHash\n              editable\n              global\n              builtin\n              inputSchema\n              jsonSchema\n              description\n              labels {\n                key\n                value\n              }\n              createdAt\n              updatedAt\n              createdBy {\n                id\n                name\n                email\n              }\n            }\n\n            fragment JobData on Job {\n              id\n              name\n              status\n              createdAt\n              createdBy {\n                id\n                name\n              }\n              startedAt\n              endedAt\n              durationMs\n              progress\n              error\n              kind\n              stages {\n                name\n                status\n                info {\n                  __typename\n                  ... on TrainingJobStageOutput {\n                    monitoringLink\n                    totalNumSamples\n                    processedNumSamples\n                    checkpoints\n                  }\n                  ... on EvalJobStageOutput {\n                    totalNumSamples\n                    processedNumSamples\n                  }\n                  ... on BatchInferenceJobStageOutput {\n                    totalNumSamples\n                    processedNumSamples\n                  }\n                }\n              }\n              useCase {\n                id\n                key\n                name\n              }\n              recipe {\n                ...CustomRecipeData\n              }\n              details {\n                args\n                recipeHash\n                artifacts {\n                  id\n                  name\n                  kind\n                  uri\n                  metadata\n                  createdAt\n                }\n              }\n            }\n            ')
        variables: Dict[str, object] = {'jobId': job_id}
        response = self.execute(query=query, operation_name='CancelJob', variables=variables, **kwargs)
        data = self.get_data(response)
        return CancelJob.model_validate(data)

    def list_datasets(self, input: str, **kwargs: Any) -> ListDatasets:
        query = gql('\n            query ListDatasets($input: IdOrKey!) {\n              datasets(useCase: $input) {\n                ...DatasetData\n              }\n            }\n\n            fragment DatasetData on Dataset {\n              id\n              key\n              name\n              createdAt\n              kind\n              records\n              metricsUsage {\n                feedbackCount\n                comparisonCount\n                metric {\n                  ...MetricData\n                }\n              }\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='ListDatasets', variables=variables, **kwargs)
        data = self.get_data(response)
        return ListDatasets.model_validate(data)

    def describe_dataset(self, input: str, usecase: str, **kwargs: Any) -> DescribeDataset:
        query = gql('\n            query DescribeDataset($input: IdOrKey!, $usecase: IdOrKey!) {\n              dataset(idOrKey: $input, useCase: $usecase) {\n                ...DatasetData\n              }\n            }\n\n            fragment DatasetData on Dataset {\n              id\n              key\n              name\n              createdAt\n              kind\n              records\n              metricsUsage {\n                feedbackCount\n                comparisonCount\n                metric {\n                  ...MetricData\n                }\n              }\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n            ')
        variables: Dict[str, object] = {'input': input, 'usecase': usecase}
        response = self.execute(query=query, operation_name='DescribeDataset', variables=variables, **kwargs)
        data = self.get_data(response)
        return DescribeDataset.model_validate(data)

    def describe_use_case(self, input: str, **kwargs: Any) -> DescribeUseCase:
        query = gql('\n            query DescribeUseCase($input: IdOrKey!) {\n              useCase(idOrKey: $input) {\n                ...UseCaseData\n              }\n            }\n\n            fragment MetricWithContextData on MetricWithContext {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n\n            fragment ModelServiceData on ModelService {\n              id\n              key\n              name\n              model {\n                ...ModelData\n                backbone {\n                  ...ModelData\n                }\n              }\n              attached\n              isDefault\n              desiredOnline\n              createdAt\n            }\n\n            fragment UseCaseData on UseCase {\n              id\n              key\n              name\n              description\n              createdAt\n              metrics {\n                ...MetricWithContextData\n              }\n              modelServices {\n                ...ModelServiceData\n              }\n              permissions\n              shares {\n                team {\n                  id\n                  key\n                  name\n                  createdAt\n                }\n                role {\n                  id\n                  key\n                  name\n                  createdAt\n                  permissions\n                }\n                isOwner\n              }\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='DescribeUseCase', variables=variables, **kwargs)
        data = self.get_data(response)
        return DescribeUseCase.model_validate(data)

    def list_use_cases(self, **kwargs: Any) -> ListUseCases:
        query = gql('\n            query ListUseCases {\n              useCases {\n                ...UseCaseData\n              }\n            }\n\n            fragment MetricWithContextData on MetricWithContext {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n\n            fragment ModelServiceData on ModelService {\n              id\n              key\n              name\n              model {\n                ...ModelData\n                backbone {\n                  ...ModelData\n                }\n              }\n              attached\n              isDefault\n              desiredOnline\n              createdAt\n            }\n\n            fragment UseCaseData on UseCase {\n              id\n              key\n              name\n              description\n              createdAt\n              metrics {\n                ...MetricWithContextData\n              }\n              modelServices {\n                ...ModelServiceData\n              }\n              permissions\n              shares {\n                team {\n                  id\n                  key\n                  name\n                  createdAt\n                }\n                role {\n                  id\n                  key\n                  name\n                  createdAt\n                  permissions\n                }\n                isOwner\n              }\n            }\n            ')
        variables: Dict[str, object] = {}
        response = self.execute(query=query, operation_name='ListUseCases', variables=variables, **kwargs)
        data = self.get_data(response)
        return ListUseCases.model_validate(data)

    def list_models(self, filter: ModelFilter, **kwargs: Any) -> ListModels:
        query = gql('\n            query ListModels($filter: ModelFilter! = {inStorage: null, available: null, trainable: null, kind: [Generation, Embedding], viewAll: false, online: null}) {\n              models(filter: $filter) {\n                ...ModelDataAdmin\n                backbone {\n                  ...ModelDataAdmin\n                }\n              }\n            }\n\n            fragment ModelDataAdmin on Model {\n              id\n              key\n              name\n              online\n              useCases {\n                id\n                key\n                name\n              }\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n            }\n            ')
        variables: Dict[str, object] = {'filter': filter}
        response = self.execute(query=query, operation_name='ListModels', variables=variables, **kwargs)
        data = self.get_data(response)
        return ListModels.model_validate(data)

    def describe_model_admin(self, input: str, **kwargs: Any) -> DescribeModelAdmin:
        query = gql('\n            query DescribeModelAdmin($input: IdOrKey!) {\n              model(idOrKey: $input) {\n                ...ModelDataAdmin\n                backbone {\n                  ...ModelDataAdmin\n                }\n              }\n            }\n\n            fragment ModelDataAdmin on Model {\n              id\n              key\n              name\n              online\n              useCases {\n                id\n                key\n                name\n              }\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='DescribeModelAdmin', variables=variables, **kwargs)
        data = self.get_data(response)
        return DescribeModelAdmin.model_validate(data)

    def describe_model(self, input: str, **kwargs: Any) -> DescribeModel:
        query = gql('\n            query DescribeModel($input: IdOrKey!) {\n              model(idOrKey: $input) {\n                ...ModelData\n                backbone {\n                  ...ModelData\n                }\n              }\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='DescribeModel', variables=variables, **kwargs)
        data = self.get_data(response)
        return DescribeModel.model_validate(data)

    def list_metrics(self, **kwargs: Any) -> ListMetrics:
        query = gql('\n            query ListMetrics {\n              metrics {\n                ...MetricDataAdmin\n              }\n            }\n\n            fragment MetricDataAdmin on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              useCases {\n                id\n                name\n                key\n                description\n              }\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n            ')
        variables: Dict[str, object] = {}
        response = self.execute(query=query, operation_name='ListMetrics', variables=variables, **kwargs)
        data = self.get_data(response)
        return ListMetrics.model_validate(data)

    def describe_metric_admin(self, input: str, **kwargs: Any) -> DescribeMetricAdmin:
        query = gql('\n            query DescribeMetricAdmin($input: IdOrKey!) {\n              metric(idOrKey: $input) {\n                ...MetricDataAdmin\n              }\n            }\n\n            fragment MetricDataAdmin on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              useCases {\n                id\n                name\n                key\n                description\n              }\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='DescribeMetricAdmin', variables=variables, **kwargs)
        data = self.get_data(response)
        return DescribeMetricAdmin.model_validate(data)

    def describe_metric(self, input: str, **kwargs: Any) -> DescribeMetric:
        query = gql('\n            query DescribeMetric($input: IdOrKey!) {\n              metric(idOrKey: $input) {\n                ...MetricData\n              }\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='DescribeMetric', variables=variables, **kwargs)
        data = self.get_data(response)
        return DescribeMetric.model_validate(data)

    def list_ab_campaigns(self, input: AbCampaignFilter, **kwargs: Any) -> ListAbCampaigns:
        query = gql('\n            query ListAbCampaigns($input: AbCampaignFilter!) {\n              abCampaigns(filter: $input) {\n                ...AbCampaignDetailData\n              }\n            }\n\n            fragment AbCampaignCreateData on Abcampaign {\n              id\n              key\n              status\n              beginDate\n            }\n\n            fragment AbCampaignDetailData on Abcampaign {\n              ...AbCampaignCreateData\n              feedbackType\n              trafficSplit\n              endDate\n              metric {\n                ...MetricData\n              }\n              useCase {\n                id\n                key\n                name\n              }\n              models {\n                id\n                key\n                name\n              }\n              feedbacks\n              hasEnoughFeedbacks\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='ListAbCampaigns', variables=variables, **kwargs)
        data = self.get_data(response)
        return ListAbCampaigns.model_validate(data)

    def describe_ab_campaign(self, input: str, **kwargs: Any) -> DescribeAbCampaign:
        query = gql('\n            query DescribeAbCampaign($input: IdOrKey!) {\n              abCampaign(idOrKey: $input) {\n                ...AbCampaignDetailData\n                report {\n                  ...AbCampaignReportData\n                }\n              }\n            }\n\n            fragment AbCampaignCreateData on Abcampaign {\n              id\n              key\n              status\n              beginDate\n            }\n\n            fragment AbCampaignDetailData on Abcampaign {\n              ...AbCampaignCreateData\n              feedbackType\n              trafficSplit\n              endDate\n              metric {\n                ...MetricData\n              }\n              useCase {\n                id\n                key\n                name\n              }\n              models {\n                id\n                key\n                name\n              }\n              feedbacks\n              hasEnoughFeedbacks\n            }\n\n            fragment AbCampaignReportData on AbReport {\n              pValue\n              variants {\n                variant {\n                  id\n                  key\n                  name\n                }\n                interval {\n                  start\n                  middle\n                  end\n                }\n                feedbacks\n                comparisons {\n                  feedbacks\n                  wins\n                  losses\n                  tiesGood\n                  tiesBad\n                  variant {\n                    id\n                    key\n                    name\n                  }\n                }\n              }\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n            ')
        variables: Dict[str, object] = {'input': input}
        response = self.execute(query=query, operation_name='DescribeAbCampaign', variables=variables, **kwargs)
        data = self.get_data(response)
        return DescribeAbCampaign.model_validate(data)

    def list_interactions(self, filter: ListCompletionsFilterInput, page: CursorPageInput, order: Union[Optional[List[OrderPair]], UnsetType]=UNSET, **kwargs: Any) -> ListInteractions:
        query = gql('\n            query ListInteractions($filter: ListCompletionsFilterInput!, $page: CursorPageInput!, $order: [OrderPair!] = [{field: "created_at", order: DESC}]) {\n              completions(filter: $filter, page: $page, order: $order) {\n                totalCount\n                pageInfo {\n                  hasNextPage\n                  endCursor\n                }\n                nodes {\n                  ...CompletionData\n                }\n              }\n            }\n\n            fragment CompletionComparisonFeedbackData on Completion {\n              id\n              completion\n              source\n              model {\n                id\n                key\n                name\n              }\n            }\n\n            fragment CompletionData on Completion {\n              id\n              chatMessages {\n                role\n                content\n              }\n              completion\n              source\n              model {\n                id\n                key\n                name\n              }\n              directFeedbacks {\n                id\n                value\n                metric {\n                  ...MetricData\n                }\n                reason\n                details\n                createdAt\n              }\n              comparisonFeedbacks {\n                id\n                createdAt\n                usecase {\n                  id\n                  key\n                  name\n                }\n                metric {\n                  id\n                  key\n                  name\n                }\n                preferedCompletion {\n                  ...CompletionComparisonFeedbackData\n                }\n                otherCompletion {\n                  ...CompletionComparisonFeedbackData\n                }\n              }\n              labels {\n                key\n                value\n              }\n              metadata {\n                parameters\n                timings\n                usage {\n                  completionTokens\n                  promptTokens\n                  totalTokens\n                }\n                system\n              }\n              createdAt\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n            ')
        variables: Dict[str, object] = {'filter': filter, 'page': page, 'order': order}
        response = self.execute(query=query, operation_name='ListInteractions', variables=variables, **kwargs)
        data = self.get_data(response)
        return ListInteractions.model_validate(data)

    def list_grouped_interactions(self, filter: ListCompletionsFilterInput, group_by: CompletionGroupBy, page: CursorPageInput, order: Union[Optional[List[OrderPair]], UnsetType]=UNSET, **kwargs: Any) -> ListGroupedInteractions:
        query = gql('\n            query ListGroupedInteractions($filter: ListCompletionsFilterInput!, $groupBy: CompletionGroupBy!, $page: CursorPageInput!, $order: [OrderPair!] = [{field: "group", order: ASC}]) {\n              completionsGrouped(\n                groupBy: $groupBy\n                filter: $filter\n                page: $page\n                order: $order\n              ) {\n                totalCount\n                groupBy\n                pageInfo {\n                  hasNextPage\n                  endCursor\n                }\n                nodes {\n                  key\n                  count\n                  directFeedbacksStats {\n                    metric {\n                      ...MetricData\n                    }\n                    feedbacks\n                    average\n                    max\n                    min\n                    stddev\n                    sum\n                  }\n                  completions(page: $page, order: [{field: "created_at", order: DESC}]) {\n                    nodes {\n                      ...CompletionData\n                    }\n                  }\n                }\n              }\n            }\n\n            fragment CompletionComparisonFeedbackData on Completion {\n              id\n              completion\n              source\n              model {\n                id\n                key\n                name\n              }\n            }\n\n            fragment CompletionData on Completion {\n              id\n              chatMessages {\n                role\n                content\n              }\n              completion\n              source\n              model {\n                id\n                key\n                name\n              }\n              directFeedbacks {\n                id\n                value\n                metric {\n                  ...MetricData\n                }\n                reason\n                details\n                createdAt\n              }\n              comparisonFeedbacks {\n                id\n                createdAt\n                usecase {\n                  id\n                  key\n                  name\n                }\n                metric {\n                  id\n                  key\n                  name\n                }\n                preferedCompletion {\n                  ...CompletionComparisonFeedbackData\n                }\n                otherCompletion {\n                  ...CompletionComparisonFeedbackData\n                }\n              }\n              labels {\n                key\n                value\n              }\n              metadata {\n                parameters\n                timings\n                usage {\n                  completionTokens\n                  promptTokens\n                  totalTokens\n                }\n                system\n              }\n              createdAt\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n            ')
        variables: Dict[str, object] = {'filter': filter, 'groupBy': group_by, 'page': page, 'order': order}
        response = self.execute(query=query, operation_name='ListGroupedInteractions', variables=variables, **kwargs)
        data = self.get_data(response)
        return ListGroupedInteractions.model_validate(data)

    def describe_interaction(self, use_case: str, id: Any, **kwargs: Any) -> DescribeInteraction:
        query = gql('\n            query DescribeInteraction($useCase: IdOrKey!, $id: UUID!) {\n              completion(useCase: $useCase, id: $id) {\n                ...CompletionData\n              }\n            }\n\n            fragment CompletionComparisonFeedbackData on Completion {\n              id\n              completion\n              source\n              model {\n                id\n                key\n                name\n              }\n            }\n\n            fragment CompletionData on Completion {\n              id\n              chatMessages {\n                role\n                content\n              }\n              completion\n              source\n              model {\n                id\n                key\n                name\n              }\n              directFeedbacks {\n                id\n                value\n                metric {\n                  ...MetricData\n                }\n                reason\n                details\n                createdAt\n              }\n              comparisonFeedbacks {\n                id\n                createdAt\n                usecase {\n                  id\n                  key\n                  name\n                }\n                metric {\n                  id\n                  key\n                  name\n                }\n                preferedCompletion {\n                  ...CompletionComparisonFeedbackData\n                }\n                otherCompletion {\n                  ...CompletionComparisonFeedbackData\n                }\n              }\n              labels {\n                key\n                value\n              }\n              metadata {\n                parameters\n                timings\n                usage {\n                  completionTokens\n                  promptTokens\n                  totalTokens\n                }\n                system\n              }\n              createdAt\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n            ')
        variables: Dict[str, object] = {'useCase': use_case, 'id': id}
        response = self.execute(query=query, operation_name='DescribeInteraction', variables=variables, **kwargs)
        data = self.get_data(response)
        return DescribeInteraction.model_validate(data)

    def list_users(self, **kwargs: Any) -> ListUsers:
        query = gql('\n            query ListUsers {\n              users {\n                ...UserData\n              }\n            }\n\n            fragment UserData on User {\n              id\n              email\n              name\n              createdAt\n              teams {\n                team {\n                  id\n                  key\n                  name\n                  createdAt\n                }\n                role {\n                  id\n                  key\n                  name\n                  createdAt\n                  permissions\n                }\n              }\n            }\n            ')
        variables: Dict[str, object] = {}
        response = self.execute(query=query, operation_name='ListUsers', variables=variables, **kwargs)
        data = self.get_data(response)
        return ListUsers.model_validate(data)

    def me(self, **kwargs: Any) -> Me:
        query = gql('\n            query Me {\n              me {\n                ...UserData\n                apiKeys {\n                  key\n                  createdAt\n                }\n              }\n            }\n\n            fragment UserData on User {\n              id\n              email\n              name\n              createdAt\n              teams {\n                team {\n                  id\n                  key\n                  name\n                  createdAt\n                }\n                role {\n                  id\n                  key\n                  name\n                  createdAt\n                  permissions\n                }\n              }\n            }\n            ')
        variables: Dict[str, object] = {}
        response = self.execute(query=query, operation_name='Me', variables=variables, **kwargs)
        data = self.get_data(response)
        return Me.model_validate(data)

    def list_teams(self, **kwargs: Any) -> ListTeams:
        query = gql('\n            query ListTeams {\n              teams {\n                id\n                key\n                name\n                createdAt\n              }\n            }\n            ')
        variables: Dict[str, object] = {}
        response = self.execute(query=query, operation_name='ListTeams', variables=variables, **kwargs)
        data = self.get_data(response)
        return ListTeams.model_validate(data)

    def list_roles(self, **kwargs: Any) -> ListRoles:
        query = gql('\n            query ListRoles {\n              roles {\n                id\n                key\n                name\n                createdAt\n                permissions\n              }\n            }\n            ')
        variables: Dict[str, object] = {}
        response = self.execute(query=query, operation_name='ListRoles', variables=variables, **kwargs)
        data = self.get_data(response)
        return ListRoles.model_validate(data)

    def list_permissions(self, **kwargs: Any) -> ListPermissions:
        query = gql('\n            query ListPermissions {\n              permissions\n            }\n            ')
        variables: Dict[str, object] = {}
        response = self.execute(query=query, operation_name='ListPermissions', variables=variables, **kwargs)
        data = self.get_data(response)
        return ListPermissions.model_validate(data)

    def list_harmony_groups(self, **kwargs: Any) -> ListHarmonyGroups:
        query = gql('\n            query ListHarmonyGroups {\n              harmonyGroups {\n                ...HarmonyGroupData\n              }\n            }\n\n            fragment HarmonyGroupData on HarmonyGroup {\n              id\n              key\n              computePool {\n                key\n                name\n              }\n              status\n              url\n              worldSize\n              gpuTypes\n              createdAt\n              onlineModels {\n                ...ModelData\n              }\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n            ')
        variables: Dict[str, object] = {}
        response = self.execute(query=query, operation_name='ListHarmonyGroups', variables=variables, **kwargs)
        data = self.get_data(response)
        return ListHarmonyGroups.model_validate(data)

    def list_compute_pools(self, **kwargs: Any) -> ListComputePools:
        query = gql('\n            query ListComputePools {\n              computePools {\n                id\n                key\n                name\n                createdAt\n                capabilities\n                harmonyGroups {\n                  ...HarmonyGroupData\n                }\n              }\n            }\n\n            fragment HarmonyGroupData on HarmonyGroup {\n              id\n              key\n              computePool {\n                key\n                name\n              }\n              status\n              url\n              worldSize\n              gpuTypes\n              createdAt\n              onlineModels {\n                ...ModelData\n              }\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n            ')
        variables: Dict[str, object] = {}
        response = self.execute(query=query, operation_name='ListComputePools', variables=variables, **kwargs)
        data = self.get_data(response)
        return ListComputePools.model_validate(data)

    def list_remote_envs(self, **kwargs: Any) -> ListRemoteEnvs:
        query = gql('\n            query ListRemoteEnvs {\n              remoteEnvs {\n                ...RemoteEnvData\n              }\n            }\n\n            fragment RemoteEnvData on RemoteEnv {\n              id\n              key\n              name\n              url\n              description\n              createdAt\n              version\n              status\n              metadataSchema\n            }\n            ')
        variables: Dict[str, object] = {}
        response = self.execute(query=query, operation_name='ListRemoteEnvs', variables=variables, **kwargs)
        data = self.get_data(response)
        return ListRemoteEnvs.model_validate(data)

    def list_custom_recipes(self, use_case: str, filter: CustomRecipeFilterInput, **kwargs: Any) -> ListCustomRecipes:
        query = gql('\n            query ListCustomRecipes($useCase: IdOrKey!, $filter: CustomRecipeFilterInput!) {\n              customRecipes(useCase: $useCase, filter: $filter) {\n                ...CustomRecipeData\n              }\n            }\n\n            fragment CustomRecipeData on CustomRecipe {\n              id\n              key\n              name\n              content\n              contentHash\n              editable\n              global\n              builtin\n              inputSchema\n              jsonSchema\n              description\n              labels {\n                key\n                value\n              }\n              createdAt\n              updatedAt\n              createdBy {\n                id\n                name\n                email\n              }\n            }\n            ')
        variables: Dict[str, object] = {'useCase': use_case, 'filter': filter}
        response = self.execute(query=query, operation_name='ListCustomRecipes', variables=variables, **kwargs)
        data = self.get_data(response)
        return ListCustomRecipes.model_validate(data)

    def get_custom_recipe(self, id_or_key: str, use_case: str, **kwargs: Any) -> GetCustomRecipe:
        query = gql('\n            query GetCustomRecipe($idOrKey: IdOrKey!, $useCase: IdOrKey!) {\n              customRecipe(idOrKey: $idOrKey, useCase: $useCase) {\n                ...CustomRecipeData\n              }\n            }\n\n            fragment CustomRecipeData on CustomRecipe {\n              id\n              key\n              name\n              content\n              contentHash\n              editable\n              global\n              builtin\n              inputSchema\n              jsonSchema\n              description\n              labels {\n                key\n                value\n              }\n              createdAt\n              updatedAt\n              createdBy {\n                id\n                name\n                email\n              }\n            }\n            ')
        variables: Dict[str, object] = {'idOrKey': id_or_key, 'useCase': use_case}
        response = self.execute(query=query, operation_name='GetCustomRecipe', variables=variables, **kwargs)
        data = self.get_data(response)
        return GetCustomRecipe.model_validate(data)

    def get_judge(self, id: str, use_case: str, version: Union[Optional[int], UnsetType]=UNSET, **kwargs: Any) -> GetJudge:
        query = gql('\n            query GetJudge($id: IdOrKey!, $useCase: IdOrKey!, $version: Int) {\n              judge(id: $id, useCase: $useCase, version: $version) {\n                ...JudgeData\n              }\n            }\n\n            fragment JudgeData on Judge {\n              id\n              key\n              version\n              name\n              criteria\n              prebuilt\n              examples {\n                input {\n                  role\n                  content\n                }\n                output\n                pass\n                reasoning\n              }\n              capabilities\n              model {\n                ...ModelData\n              }\n              useCaseId\n              metric {\n                ...MetricData\n              }\n              createdAt\n              updatedAt\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n            ')
        variables: Dict[str, object] = {'id': id, 'useCase': use_case, 'version': version}
        response = self.execute(query=query, operation_name='GetJudge', variables=variables, **kwargs)
        data = self.get_data(response)
        return GetJudge.model_validate(data)

    def list_judges(self, use_case: str, **kwargs: Any) -> ListJudges:
        query = gql('\n            query ListJudges($useCase: IdOrKey!) {\n              judges(useCase: $useCase) {\n                ...JudgeData\n              }\n            }\n\n            fragment JudgeData on Judge {\n              id\n              key\n              version\n              name\n              criteria\n              prebuilt\n              examples {\n                input {\n                  role\n                  content\n                }\n                output\n                pass\n                reasoning\n              }\n              capabilities\n              model {\n                ...ModelData\n              }\n              useCaseId\n              metric {\n                ...MetricData\n              }\n              createdAt\n              updatedAt\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n            ')
        variables: Dict[str, object] = {'useCase': use_case}
        response = self.execute(query=query, operation_name='ListJudges', variables=variables, **kwargs)
        data = self.get_data(response)
        return ListJudges.model_validate(data)

    def list_judge_versions(self, use_case: str, key: str, **kwargs: Any) -> ListJudgeVersions:
        query = gql('\n            query ListJudgeVersions($useCase: IdOrKey!, $key: String!) {\n              judgeVersions(useCase: $useCase, key: $key) {\n                ...JudgeData\n              }\n            }\n\n            fragment JudgeData on Judge {\n              id\n              key\n              version\n              name\n              criteria\n              prebuilt\n              examples {\n                input {\n                  role\n                  content\n                }\n                output\n                pass\n                reasoning\n              }\n              capabilities\n              model {\n                ...ModelData\n              }\n              useCaseId\n              metric {\n                ...MetricData\n              }\n              createdAt\n              updatedAt\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n            ')
        variables: Dict[str, object] = {'useCase': use_case, 'key': key}
        response = self.execute(query=query, operation_name='ListJudgeVersions', variables=variables, **kwargs)
        data = self.get_data(response)
        return ListJudgeVersions.model_validate(data)

    def get_grader(self, id: str, use_case: str, **kwargs: Any) -> GetGrader:
        query = gql('\n            query GetGrader($id: IdOrKey!, $useCase: IdOrKey!) {\n              grader(id: $id, useCase: $useCase) {\n                ...GraderData\n              }\n            }\n\n            fragment GraderData on Grader {\n              id\n              name\n              key\n              locked\n              graderType\n              graderConfig {\n                __typename\n                ... on JudgeConfigOutput {\n                  judgeCriteria: criteria\n                  examples {\n                    input {\n                      role\n                      content\n                    }\n                    output\n                    pass\n                    reasoning\n                  }\n                  model {\n                    ...ModelData\n                  }\n                }\n                ... on PrebuiltConfigOutput {\n                  prebuiltCriteria: criteria {\n                    key\n                    name\n                    feedbackKey\n                    description\n                  }\n                  model {\n                    ...ModelData\n                  }\n                }\n                ... on RemoteConfigOutput {\n                  url\n                  version\n                  description\n                }\n                ... on CustomConfigOutput {\n                  graderDescription: description\n                }\n              }\n              useCase {\n                id\n                key\n                name\n              }\n              metric {\n                ...MetricData\n              }\n              createdAt\n              updatedAt\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n            ')
        variables: Dict[str, object] = {'id': id, 'useCase': use_case}
        response = self.execute(query=query, operation_name='GetGrader', variables=variables, **kwargs)
        data = self.get_data(response)
        return GetGrader.model_validate(data)

    def list_graders(self, use_case: str, **kwargs: Any) -> ListGraders:
        query = gql('\n            query ListGraders($useCase: IdOrKey!) {\n              graders(useCase: $useCase) {\n                ...GraderData\n              }\n            }\n\n            fragment GraderData on Grader {\n              id\n              name\n              key\n              locked\n              graderType\n              graderConfig {\n                __typename\n                ... on JudgeConfigOutput {\n                  judgeCriteria: criteria\n                  examples {\n                    input {\n                      role\n                      content\n                    }\n                    output\n                    pass\n                    reasoning\n                  }\n                  model {\n                    ...ModelData\n                  }\n                }\n                ... on PrebuiltConfigOutput {\n                  prebuiltCriteria: criteria {\n                    key\n                    name\n                    feedbackKey\n                    description\n                  }\n                  model {\n                    ...ModelData\n                  }\n                }\n                ... on RemoteConfigOutput {\n                  url\n                  version\n                  description\n                }\n                ... on CustomConfigOutput {\n                  graderDescription: description\n                }\n              }\n              useCase {\n                id\n                key\n                name\n              }\n              metric {\n                ...MetricData\n              }\n              createdAt\n              updatedAt\n            }\n\n            fragment MetricData on Metric {\n              id\n              key\n              name\n              kind\n              description\n              scoringType\n              createdAt\n              hasDirectFeedbacks\n              hasComparisonFeedbacks\n            }\n\n            fragment ModelData on Model {\n              id\n              key\n              name\n              online\n              isExternal\n              providerName\n              isAdapter\n              isTraining\n              createdAt\n              kind\n              size\n              computeConfig {\n                tp\n                kvCacheLen\n                maxSeqLen\n              }\n            }\n            ')
        variables: Dict[str, object] = {'useCase': use_case}
        response = self.execute(query=query, operation_name='ListGraders', variables=variables, **kwargs)
        data = self.get_data(response)
        return ListGraders.model_validate(data)

    def list_jobs(self, page: CursorPageInput, filter: Union[Optional[ListJobsFilterInput], UnsetType]=UNSET, order: Union[Optional[List[OrderPair]], UnsetType]=UNSET, **kwargs: Any) -> ListJobs:
        query = gql('\n            query ListJobs($page: CursorPageInput!, $filter: ListJobsFilterInput, $order: [OrderPair!]) {\n              jobs(page: $page, filter: $filter, order: $order) {\n                totalCount\n                pageInfo {\n                  hasNextPage\n                  hasPreviousPage\n                  startCursor\n                  endCursor\n                }\n                nodes {\n                  ...JobData\n                }\n              }\n            }\n\n            fragment CustomRecipeData on CustomRecipe {\n              id\n              key\n              name\n              content\n              contentHash\n              editable\n              global\n              builtin\n              inputSchema\n              jsonSchema\n              description\n              labels {\n                key\n                value\n              }\n              createdAt\n              updatedAt\n              createdBy {\n                id\n                name\n                email\n              }\n            }\n\n            fragment JobData on Job {\n              id\n              name\n              status\n              createdAt\n              createdBy {\n                id\n                name\n              }\n              startedAt\n              endedAt\n              durationMs\n              progress\n              error\n              kind\n              stages {\n                name\n                status\n                info {\n                  __typename\n                  ... on TrainingJobStageOutput {\n                    monitoringLink\n                    totalNumSamples\n                    processedNumSamples\n                    checkpoints\n                  }\n                  ... on EvalJobStageOutput {\n                    totalNumSamples\n                    processedNumSamples\n                  }\n                  ... on BatchInferenceJobStageOutput {\n                    totalNumSamples\n                    processedNumSamples\n                  }\n                }\n              }\n              useCase {\n                id\n                key\n                name\n              }\n              recipe {\n                ...CustomRecipeData\n              }\n              details {\n                args\n                recipeHash\n                artifacts {\n                  id\n                  name\n                  kind\n                  uri\n                  metadata\n                  createdAt\n                }\n              }\n            }\n            ')
        variables: Dict[str, object] = {'page': page, 'filter': filter, 'order': order}
        response = self.execute(query=query, operation_name='ListJobs', variables=variables, **kwargs)
        data = self.get_data(response)
        return ListJobs.model_validate(data)

    def describe_job(self, id: Any, **kwargs: Any) -> DescribeJob:
        query = gql('\n            query DescribeJob($id: UUID!) {\n              job(id: $id) {\n                ...JobData\n              }\n            }\n\n            fragment CustomRecipeData on CustomRecipe {\n              id\n              key\n              name\n              content\n              contentHash\n              editable\n              global\n              builtin\n              inputSchema\n              jsonSchema\n              description\n              labels {\n                key\n                value\n              }\n              createdAt\n              updatedAt\n              createdBy {\n                id\n                name\n                email\n              }\n            }\n\n            fragment JobData on Job {\n              id\n              name\n              status\n              createdAt\n              createdBy {\n                id\n                name\n              }\n              startedAt\n              endedAt\n              durationMs\n              progress\n              error\n              kind\n              stages {\n                name\n                status\n                info {\n                  __typename\n                  ... on TrainingJobStageOutput {\n                    monitoringLink\n                    totalNumSamples\n                    processedNumSamples\n                    checkpoints\n                  }\n                  ... on EvalJobStageOutput {\n                    totalNumSamples\n                    processedNumSamples\n                  }\n                  ... on BatchInferenceJobStageOutput {\n                    totalNumSamples\n                    processedNumSamples\n                  }\n                }\n              }\n              useCase {\n                id\n                key\n                name\n              }\n              recipe {\n                ...CustomRecipeData\n              }\n              details {\n                args\n                recipeHash\n                artifacts {\n                  id\n                  name\n                  kind\n                  uri\n                  metadata\n                  createdAt\n                }\n              }\n            }\n            ')
        variables: Dict[str, object] = {'id': id}
        response = self.execute(query=query, operation_name='DescribeJob', variables=variables, **kwargs)
        data = self.get_data(response)
        return DescribeJob.model_validate(data)

    def execute_custom_operation(self, *fields: GraphQLField, operation_type: OperationType, operation_name: str) -> Dict[str, Any]:
        selections = self._build_selection_set(fields)
        combined_variables = self._combine_variables(fields)
        variable_definitions = self._build_variable_definitions(combined_variables['types'])
        operation_ast = self._build_operation_ast(selections, operation_type, operation_name, variable_definitions)
        response = self.execute(print_ast(operation_ast), variables=combined_variables['values'], operation_name=operation_name)
        return self.get_data(response)

    def _combine_variables(self, fields: Tuple[GraphQLField, ...]) -> Dict[str, Dict[str, Any]]:
        variables_types_combined = {}
        processed_variables_combined = {}
        for field in fields:
            formatted_variables = field.get_formatted_variables()
            variables_types_combined.update({k: v['type'] for (k, v) in formatted_variables.items()})
            processed_variables_combined.update({k: v['value'] for (k, v) in formatted_variables.items()})
        return {'types': variables_types_combined, 'values': processed_variables_combined}

    def _build_variable_definitions(self, variables_types_combined: Dict[str, str]) -> List[VariableDefinitionNode]:
        return [VariableDefinitionNode(variable=VariableNode(name=NameNode(value=var_name)), type=NamedTypeNode(name=NameNode(value=var_value))) for (var_name, var_value) in variables_types_combined.items()]

    def _build_operation_ast(self, selections: List[SelectionNode], operation_type: OperationType, operation_name: str, variable_definitions: List[VariableDefinitionNode]) -> DocumentNode:
        return DocumentNode(definitions=[OperationDefinitionNode(operation=operation_type, name=NameNode(value=operation_name), variable_definitions=variable_definitions, selection_set=SelectionSetNode(selections=selections))])

    def _build_selection_set(self, fields: Tuple[GraphQLField, ...]) -> List[SelectionNode]:
        return [field.to_ast(idx) for (idx, field) in enumerate(fields)]

    def query(self, *fields: GraphQLField, operation_name: str) -> Dict[str, Any]:
        return self.execute_custom_operation(*fields, operation_type=OperationType.QUERY, operation_name=operation_name)

    def mutation(self, *fields: GraphQLField, operation_name: str) -> Dict[str, Any]:
        return self.execute_custom_operation(*fields, operation_type=OperationType.MUTATION, operation_name=operation_name)


from .reward_client import RewardClient
from .reward_server import RewardServer, EmptyMetadata
from .reward_types import (
    Request,
    ValidatedRequest,
    Response,
    BatchedRequest,
    BatchedResponse,
    ServerInfo,
    Turn,
)

__all__ = [
    "RewardClient",
    "RewardServer",
    "EmptyMetadata",
    "ValidatedRequest",
    "Request",
    "Response",
    "BatchedRequest",
    "BatchedResponse",
    "ServerInfo",
    "Turn",
]



import asyncio
import json
import os
import tempfile
from typing import Dict
from uuid import uuid4
from fastapi import APIRouter, FastAPI
from fastapi.responses import JSONResponse
import httpx
import portalocker
from datetime import datetime, timezone

class RequestsJournal:
    def __init__(
        self,
        max_requests_journal_size: int = 1000,
        journal_file_path: str | None = None,
    ):
        assert max_requests_journal_size > 0
        if journal_file_path is None:
            journal_file_path = os.path.join(
                tempfile.gettempdir(), "requests.journal.jsonl"
            )
        self.journal_file_path = journal_file_path
        self.max_requests_journal_size = max_requests_journal_size

    def _rotate_journal(self):
        if not os.path.exists(self.journal_file_path):
            return

        with open(self.journal_file_path + ".lock", "w", encoding="utf-8") as lock_f:
            portalocker.lock(lock_f, portalocker.LockFlags.EXCLUSIVE)

            # Safe to rotate now
            with open(self.journal_file_path, "r", encoding="utf-8") as f:
                lines = f.readlines()

            if len(lines) > self.max_requests_journal_size:
                lines = lines[-self.max_requests_journal_size :]
                with open(self.journal_file_path, "w", encoding="utf-8") as f:
                    f.writelines(lines)

            portalocker.unlock(lock_f)

    def add_journalling(self, app: FastAPI):
        router = APIRouter()
        router.add_api_route("/requests", self.list_requests, methods=["GET"])
        router.add_api_route(
            "/requests/replay_last/{n}", self.replay_last_n, methods=["POST"]
        )

        app.include_router(router=router)

        @app.on_event("startup")
        async def start_flusher():
            asyncio.create_task(self._periodic_rotate_journal_file())

        @app.middleware("http")
        async def record_requests(request, call_next):
            # don't keep track of those requests
            if request.url.path.startswith("/requests") or request.url.path.startswith(
                "/info"
            ):
                return await call_next(request)

            body = await request.body()
            req_id = str(uuid4())

            record = {
                "method": request.method,
                "url": str(request.url),
                "path": request.url.path,
                "headers": dict(request.headers),
                "body": body.decode("utf-8", errors="ignore"),
            }

            # journal request before running it
            self._journal_request(req_id, record)
            response = await call_next(request)
            return response

    def _journal_request(self, req_id: str, record: Dict):
        now = datetime.now(timezone.utc).isoformat()
        entry = {"id": req_id, "req_date": now, **record}

        with open(self.journal_file_path, "a", encoding="utf-8") as f:
            portalocker.lock(f, portalocker.LockFlags.EXCLUSIVE)
            f.write(json.dumps(entry) + "\n")
            f.flush()
            portalocker.unlock(f)

    async def _periodic_rotate_journal_file(self):
        while True:
            await asyncio.sleep(60)
            self._rotate_journal()

    def _read_journal_tail(self, n: int):
        if not os.path.exists(self.journal_file_path):
            return []
        with open(self.journal_file_path, "r", encoding="utf-8") as f:
            lines = f.readlines()
            lines = lines[-n:]
            return [json.loads(line) for line in lines]

    async def list_requests(self):
        recent = self._read_journal_tail(self.max_requests_journal_size)
        return recent

    async def replay_last_n(self, n: int):
        if n > self.max_requests_journal_size:
            return JSONResponse(
                status_code=400,
                content={"error": f"Max is {self.max_requests_journal_size}"},
            )

        entries = self._read_journal_tail(n)
        responses = []

        async with httpx.AsyncClient() as client:
            for entry in entries:
                response = await client.request(
                    method=entry["method"],
                    url=entry["url"],
                    headers=entry["headers"],
                    content=entry["body"],
                )
                responses.append(
                    {
                        "status_code": response.status_code,
                        "url": entry["url"],
                        "body": response.text,
                        "request_id": entry["id"],
                    }
                )

        return responses



from typing import Optional
from .base_model import BaseModel
from .fragments import JobData

class DescribeJob(BaseModel):
    """@public"""
    job: Optional['DescribeJobJob']

class DescribeJobJob(JobData):
    """@public"""
    pass
DescribeJob.model_rebuild()


from __future__ import annotations
import os
import io
import zipfile
import mimetypes
from contextlib import contextmanager
from loguru import logger
from hypothesis_jsonschema import from_schema
from typing import TYPE_CHECKING, Sequence, Any
from pathlib import Path

from adaptive_sdk.graphql_client.fragments import JobData
from adaptive_sdk.graphql_client.input_types import JobInput

from .base_resource import SyncAPIResource, AsyncAPIResource, UseCaseResource
from adaptive_sdk.graphql_client import (
    CustomRecipeData,
    CustomRecipeFilterInput,
    CreateRecipeInput,
    UpdateRecipeInput,
    LabelInput,
    Upload,
)

if TYPE_CHECKING:
    from adaptive_sdk.client import Adaptive, AsyncAdaptive


class Recipes(SyncAPIResource, UseCaseResource):  # type: ignore[misc]
    """
    Resource to interact with custom scripts.
    """

    def __init__(self, client: Adaptive) -> None:
        SyncAPIResource.__init__(self, client)
        UseCaseResource.__init__(self, client)

    def list(self, use_case: str | None = None) -> Sequence[CustomRecipeData]:
        filter = CustomRecipeFilterInput()
        return self._gql_client.list_custom_recipes(use_case=self.use_case_key(use_case), filter=filter).custom_recipes

    def upload(
        self,
        path: str,
        recipe_key: str,
        name: str | None = None,
        description: str | None = None,
        labels: dict[str, str] | None = None,
        use_case: str | None = None,
    ) -> CustomRecipeData:
        """
        Upload a recipe from either a single Python file or a directory (path).
        If a directory is provided, it must contain a 'main.py' and will be zipped in-memory before upload.
        """
        inferred_name = name or recipe_key
        label_inputs = [LabelInput(key=k, value=v) for k, v in labels.items()] if labels else None
        input = CreateRecipeInput(
            key=recipe_key,
            name=inferred_name,
            description=description,
            labels=label_inputs,
        )
        with _upload_from_path(path) as file_upload:
            return self._gql_client.create_custom_recipe(
                use_case=self.use_case_key(use_case), input=input, file=file_upload
            ).create_custom_recipe

    def get(
        self,
        recipe_key: str,
        use_case: str | None = None,
    ) -> CustomRecipeData | None:
        return self._gql_client.get_custom_recipe(
            id_or_key=recipe_key, use_case=self.use_case_key(use_case)
        ).custom_recipe

    def update(
        self,
        recipe_key: str,
        path: str | None = None,
        name: str | None = None,
        description: str | None = None,
        labels: Sequence[tuple[str, str]] | None = None,
        use_case: str | None = None,
    ) -> CustomRecipeData:
        label_inputs = [LabelInput(key=k, value=v) for k, v in labels] if labels else None
        input = UpdateRecipeInput(
            name=name,
            description=description,
            labels=label_inputs,
        )

        if path:
            with _upload_from_path(path) as file_upload:
                return self._gql_client.update_custom_recipe(
                    use_case=self.use_case_key(use_case),
                    id=recipe_key,
                    input=input,
                    file=file_upload,
                ).update_custom_recipe
        else:
            return self._gql_client.update_custom_recipe(
                use_case=self.use_case_key(use_case),
                id=recipe_key,
                input=input,
                file=None,
            ).update_custom_recipe

    def delete(
        self,
        recipe_key: str,
        use_case: str | None = None,
    ) -> bool:
        return self._gql_client.delete_custom_recipe(
            use_case=self.use_case_key(use_case), id=recipe_key
        ).delete_custom_recipe

    def generate_sample_input(self, recipe_key: str, use_case: str | None = None) -> dict:
        recipe_details = self.get(recipe_key=recipe_key, use_case=self.use_case_key(use_case))
        if recipe_details is None:
            raise ValueError(f"Recipe {recipe_key} was not found")
        strategy = from_schema(recipe_details.json_schema)

        best_example = None
        max_key_count = -1

        for _ in range(10):
            try:
                example = strategy.example()
                current_key_count = _count_keys_recursively(example)

                if current_key_count > max_key_count:
                    max_key_count = current_key_count
                    best_example = example
            except Exception as e:
                print(f"Warning: Failed to generate an example due to: {e}")
                # Continue to next iteration even if one example fails

        if best_example is None:
            print("A valid sample could not be generated. Returning an empty dict.")
            best_example = {}
        return dict(best_example)  # type: ignore


class AsyncRecipes(AsyncAPIResource, UseCaseResource):  # type: ignore[misc]
    """
    Resource to interact with custom scripts.
    """

    def __init__(self, client: AsyncAdaptive) -> None:
        AsyncAPIResource.__init__(self, client)
        UseCaseResource.__init__(self, client)

    async def list(self, use_case: str | None = None) -> Sequence[CustomRecipeData]:
        filter = CustomRecipeFilterInput()
        return (
            await self._gql_client.list_custom_recipes(use_case=self.use_case_key(use_case), filter=filter)
        ).custom_recipes

    async def upload(
        self,
        path: str,
        recipe_key: str,
        name: str | None = None,
        description: str | None = None,
        labels: Sequence[tuple[str, str]] | None = None,
        use_case: str | None = None,
    ) -> CustomRecipeData:
        inferred_name = name or recipe_key
        label_inputs = [LabelInput(key=k, value=v) for k, v in labels] if labels else None
        input = CreateRecipeInput(
            key=recipe_key,
            name=inferred_name,
            description=description,
            labels=label_inputs,
        )
        with _upload_from_path(path) as file_upload:
            return (
                await self._gql_client.create_custom_recipe(
                    use_case=self.use_case_key(use_case), input=input, file=file_upload
                )
            ).create_custom_recipe

    async def get(
        self,
        recipe_key: str,
        use_case: str | None = None,
    ) -> CustomRecipeData | None:
        return (
            await self._gql_client.get_custom_recipe(id_or_key=recipe_key, use_case=self.use_case_key(use_case))
        ).custom_recipe

    async def update(
        self,
        recipe_key: str,
        path: str | None = None,
        name: str | None = None,
        description: str | None = None,
        labels: Sequence[tuple[str, str]] | None = None,
        use_case: str | None = None,
    ) -> CustomRecipeData:
        label_inputs = [LabelInput(key=k, value=v) for k, v in labels] if labels else None
        input = UpdateRecipeInput(
            name=name,
            description=description,
            labels=label_inputs,
        )

        if path:
            with _upload_from_path(path) as file_upload:
                return (
                    await self._gql_client.update_custom_recipe(
                        use_case=self.use_case_key(use_case),
                        id=recipe_key,
                        input=input,
                        file=file_upload,
                    )
                ).update_custom_recipe
        else:
            return (
                await self._gql_client.update_custom_recipe(
                    use_case=self.use_case_key(use_case),
                    id=recipe_key,
                    input=input,
                    file=None,
                )
            ).update_custom_recipe

    async def delete(
        self,
        recipe_key: str,
        use_case: str | None = None,
    ) -> bool:
        return (
            await self._gql_client.delete_custom_recipe(use_case=self.use_case_key(use_case), id=recipe_key)
        ).delete_custom_recipe

    async def generate_sample_input(self, recipe_key: str, use_case: str | None = None) -> dict:
        recipe_details = await self.get(recipe_key=recipe_key, use_case=self.use_case_key(use_case))
        if recipe_details is None:
            raise ValueError(f"Recipe {recipe_key} was not found")
        strategy = from_schema(recipe_details.json_schema)

        best_example = None
        max_key_count = -1

        for _ in range(10):
            try:
                example = strategy.example()
                current_key_count = _count_keys_recursively(example)

                if current_key_count > max_key_count:
                    max_key_count = current_key_count
                    best_example = example
            except Exception as e:
                print(f"Warning: Failed to generate an example due to: {e}")
                # Continue to next iteration even if one example fails

        if best_example is None:
            print("A valid sample could not be generated. Returning an empty dict.")
            best_example = {}
        return dict(best_example)  # type: ignore


def _count_keys_recursively(data: Any) -> int:
    """Recursively counts the total number of keys in dictionaries within the data."""
    count = 0
    if isinstance(data, dict):
        count += len(data)
        for value in data.values():
            count += _count_keys_recursively(value)
    elif isinstance(data, list):
        for item in data:
            count += _count_keys_recursively(item)
    return count


def _validate_python_file(path: Path) -> None:
    """Validate that the path exists, is a file and has a .py extension."""
    if not path.exists():
        raise FileNotFoundError(f"Python file not found: {path}")
    if not path.is_file():
        raise ValueError(f"Expected a file path, got a directory or non-file: {path}")
    if path.suffix.lower() != ".py":
        raise ValueError(f"Expected a Python file with .py extension, got: {path}")


def _validate_recipe_directory(dir_path: Path) -> None:
    """Validate that the directory exists and contains a main.py file."""
    if not dir_path.exists():
        raise FileNotFoundError(f"Directory not found: {dir_path}")
    if not dir_path.is_dir():
        raise ValueError(f"Expected a directory path, got a file: {dir_path}")
    main_py = dir_path / "main.py"
    if not main_py.exists() or not main_py.is_file():
        raise FileNotFoundError(f"Directory must contain a 'main.py' file: {dir_path}")


def _zip_directory_to_bytes_io(dir_path: Path) -> io.BytesIO:
    """Zip the contents of a directory into an in-memory BytesIO buffer."""
    buffer = io.BytesIO()
    with zipfile.ZipFile(buffer, mode="w", compression=zipfile.ZIP_DEFLATED) as zf:
        for root, _, files in os.walk(dir_path):
            for file_name in files:
                file_path = Path(root) / file_name
                arcname = file_path.relative_to(dir_path)
                zf.write(file_path, arcname.as_posix())
    buffer.seek(0)
    return buffer


@contextmanager
def _upload_from_path(path: str):
    """
    Context manager yielding an Upload object for a Python file or a directory.

    - If path is a .py file, validates and opens it for upload.
    - If path is a directory, validates it contains main.py, zips contents in-memory.
    """
    p = Path(path)
    if p.is_file():
        _validate_python_file(p)
        filename = p.name
        content_type = mimetypes.guess_type(str(p))[0] or "application/octet-stream"
        f = open(p, "rb")
        try:
            yield Upload(filename=filename, content=f, content_type=content_type)
        finally:
            f.close()
    elif p.is_dir():
        _validate_recipe_directory(p)
        # Ensure __init__.py exists at the root of the directory before zipping
        created_init = False
        root_init = p / "__init__.py"
        zip_buffer = None
        try:
            if not root_init.exists():
                root_init.touch()
                created_init = True
                logger.info(f"Added __init__.py to your directory, as it is required for proper execution of recipe")
            zip_buffer = _zip_directory_to_bytes_io(p)
        finally:
            if created_init:
                try:
                    root_init.unlink()
                    logger.info(f"Cleaned up __init__.py from your directory")
                except Exception:
                    logger.error(f"Failed to remove __init__.py from your directory")
                    pass
        if zip_buffer is None:
            raise RuntimeError("Failed to create in-memory zip for directory upload")

        filename = f"{p.name}.zip"
        try:
            yield Upload(filename=filename, content=zip_buffer, content_type="application/zip")
        finally:
            zip_buffer.close()
    else:
        if not p.exists():
            raise FileNotFoundError(f"Path not found: {path}")
        raise ValueError(f"Path must be a Python file or a directory: {path}")



from typing import List
from pydantic import Field
from .base_model import BaseModel
from .fragments import AbCampaignDetailData

class ListAbCampaigns(BaseModel):
    """@public"""
    ab_campaigns: List['ListAbCampaignsAbCampaigns'] = Field(alias='abCampaigns')

class ListAbCampaignsAbCampaigns(AbCampaignDetailData):
    """@public"""
    pass
ListAbCampaigns.model_rebuild()


from typing import Annotated, Any, List, Literal, Optional, Union
from pydantic import Field
from .base_model import BaseModel
from .enums import AbcampaignStatus, CompletionSource, DatasetKind, FeedbackType, GraderTypeEnum, HarmonyStatus, JobArtifactKind, JobKind, JobStatus, JobStatusOutput, JudgeCapability, MetricKind, MetricScoringType, ModelKindFilter, ModelOnline, PrebuiltCriteriaKey, ProviderName, RemoteEnvStatus

class AbCampaignCreateData(BaseModel):
    """@public"""
    id: Any
    key: str
    status: AbcampaignStatus
    begin_date: int = Field(alias='beginDate')

class MetricData(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str
    kind: MetricKind
    description: str
    scoring_type: MetricScoringType = Field(alias='scoringType')
    created_at: int = Field(alias='createdAt')
    has_direct_feedbacks: bool = Field(alias='hasDirectFeedbacks')
    has_comparison_feedbacks: bool = Field(alias='hasComparisonFeedbacks')

class AbCampaignDetailData(AbCampaignCreateData):
    """@public"""
    feedback_type: FeedbackType = Field(alias='feedbackType')
    traffic_split: float = Field(alias='trafficSplit')
    end_date: Optional[int] = Field(alias='endDate')
    metric: Optional['AbCampaignDetailDataMetric']
    use_case: Optional['AbCampaignDetailDataUseCase'] = Field(alias='useCase')
    models: List['AbCampaignDetailDataModels']
    feedbacks: int
    has_enough_feedbacks: bool = Field(alias='hasEnoughFeedbacks')

class AbCampaignDetailDataMetric(MetricData):
    """@public"""
    pass

class AbCampaignDetailDataUseCase(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str

class AbCampaignDetailDataModels(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str

class AbCampaignReportData(BaseModel):
    """@public"""
    p_value: Optional[float] = Field(alias='pValue')
    variants: List['AbCampaignReportDataVariants']

class AbCampaignReportDataVariants(BaseModel):
    """@public"""
    variant: 'AbCampaignReportDataVariantsVariant'
    interval: Optional['AbCampaignReportDataVariantsInterval']
    feedbacks: int
    comparisons: Optional[List['AbCampaignReportDataVariantsComparisons']]

class AbCampaignReportDataVariantsVariant(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str

class AbCampaignReportDataVariantsInterval(BaseModel):
    """@public"""
    start: float
    middle: float
    end: float

class AbCampaignReportDataVariantsComparisons(BaseModel):
    """@public"""
    feedbacks: int
    wins: int
    losses: int
    ties_good: int = Field(alias='tiesGood')
    ties_bad: int = Field(alias='tiesBad')
    variant: 'AbCampaignReportDataVariantsComparisonsVariant'

class AbCampaignReportDataVariantsComparisonsVariant(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str

class CompletionComparisonFeedbackData(BaseModel):
    """@public"""
    id: Any
    completion: Optional[str]
    source: CompletionSource
    model: Optional['CompletionComparisonFeedbackDataModel']

class CompletionComparisonFeedbackDataModel(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str

class CompletionData(BaseModel):
    """@public"""
    id: Any
    chat_messages: List['CompletionDataChatMessages'] = Field(alias='chatMessages')
    completion: Optional[str]
    source: CompletionSource
    model: Optional['CompletionDataModel']
    direct_feedbacks: List['CompletionDataDirectFeedbacks'] = Field(alias='directFeedbacks')
    comparison_feedbacks: List['CompletionDataComparisonFeedbacks'] = Field(alias='comparisonFeedbacks')
    labels: List['CompletionDataLabels']
    metadata: 'CompletionDataMetadata'
    created_at: int = Field(alias='createdAt')

class CompletionDataChatMessages(BaseModel):
    """@public"""
    role: str
    content: str

class CompletionDataModel(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str

class CompletionDataDirectFeedbacks(BaseModel):
    """@public"""
    id: Any
    value: float
    metric: Optional['CompletionDataDirectFeedbacksMetric']
    reason: Optional[str]
    details: Optional[str]
    created_at: int = Field(alias='createdAt')

class CompletionDataDirectFeedbacksMetric(MetricData):
    """@public"""
    pass

class CompletionDataComparisonFeedbacks(BaseModel):
    """@public"""
    id: Any
    created_at: int = Field(alias='createdAt')
    usecase: Optional['CompletionDataComparisonFeedbacksUsecase']
    metric: Optional['CompletionDataComparisonFeedbacksMetric']
    prefered_completion: Optional['CompletionDataComparisonFeedbacksPreferedCompletion'] = Field(alias='preferedCompletion')
    other_completion: Optional['CompletionDataComparisonFeedbacksOtherCompletion'] = Field(alias='otherCompletion')

class CompletionDataComparisonFeedbacksUsecase(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str

class CompletionDataComparisonFeedbacksMetric(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str

class CompletionDataComparisonFeedbacksPreferedCompletion(CompletionComparisonFeedbackData):
    """@public"""
    pass

class CompletionDataComparisonFeedbacksOtherCompletion(CompletionComparisonFeedbackData):
    """@public"""
    pass

class CompletionDataLabels(BaseModel):
    """@public"""
    key: str
    value: str

class CompletionDataMetadata(BaseModel):
    """@public"""
    parameters: Optional[Any]
    timings: Optional[Any]
    usage: Optional['CompletionDataMetadataUsage']
    system: Optional[Any]

class CompletionDataMetadataUsage(BaseModel):
    """@public"""
    completion_tokens: int = Field(alias='completionTokens')
    prompt_tokens: int = Field(alias='promptTokens')
    total_tokens: int = Field(alias='totalTokens')

class CustomRecipeData(BaseModel):
    """@public"""
    id: Any
    key: Optional[str]
    name: str
    content: str
    content_hash: str = Field(alias='contentHash')
    editable: bool
    global_: bool = Field(alias='global')
    builtin: bool
    input_schema: Any = Field(alias='inputSchema')
    json_schema: Any = Field(alias='jsonSchema')
    description: str
    labels: List['CustomRecipeDataLabels']
    created_at: int = Field(alias='createdAt')
    updated_at: Optional[int] = Field(alias='updatedAt')
    created_by: Optional['CustomRecipeDataCreatedBy'] = Field(alias='createdBy')

class CustomRecipeDataLabels(BaseModel):
    """@public"""
    key: str
    value: str

class CustomRecipeDataCreatedBy(BaseModel):
    """@public"""
    id: Any
    name: str
    email: str

class DatasetData(BaseModel):
    """@public"""
    id: Any
    key: Optional[str]
    name: str
    created_at: Any = Field(alias='createdAt')
    kind: DatasetKind
    records: Optional[int]
    metrics_usage: List['DatasetDataMetricsUsage'] = Field(alias='metricsUsage')

class DatasetDataMetricsUsage(BaseModel):
    """@public"""
    feedback_count: int = Field(alias='feedbackCount')
    comparison_count: int = Field(alias='comparisonCount')
    metric: 'DatasetDataMetricsUsageMetric'

class DatasetDataMetricsUsageMetric(MetricData):
    """@public"""
    pass

class ModelData(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str
    online: ModelOnline
    is_external: bool = Field(alias='isExternal')
    provider_name: ProviderName = Field(alias='providerName')
    is_adapter: bool = Field(alias='isAdapter')
    is_training: bool = Field(alias='isTraining')
    created_at: int = Field(alias='createdAt')
    kind: ModelKindFilter
    size: Optional[str]
    compute_config: Optional['ModelDataComputeConfig'] = Field(alias='computeConfig')

class ModelDataComputeConfig(BaseModel):
    """@public"""
    tp: int
    kv_cache_len: int = Field(alias='kvCacheLen')
    max_seq_len: int = Field(alias='maxSeqLen')

class GraderData(BaseModel):
    """@public"""
    id: Any
    name: str
    key: str
    locked: bool
    grader_type: GraderTypeEnum = Field(alias='graderType')
    grader_config: Union['GraderDataGraderConfigJudgeConfigOutput', 'GraderDataGraderConfigPrebuiltConfigOutput', 'GraderDataGraderConfigRemoteConfigOutput', 'GraderDataGraderConfigCustomConfigOutput'] = Field(alias='graderConfig', discriminator='typename__')
    use_case: 'GraderDataUseCase' = Field(alias='useCase')
    metric: 'GraderDataMetric'
    created_at: int = Field(alias='createdAt')
    updated_at: int = Field(alias='updatedAt')

class GraderDataGraderConfigJudgeConfigOutput(BaseModel):
    """@public"""
    typename__: Literal['JudgeConfigOutput'] = Field(alias='__typename')
    judge_criteria: str = Field(alias='judgeCriteria')
    examples: List['GraderDataGraderConfigJudgeConfigOutputExamples']
    model: 'GraderDataGraderConfigJudgeConfigOutputModel'

class GraderDataGraderConfigJudgeConfigOutputExamples(BaseModel):
    """@public"""
    input: List['GraderDataGraderConfigJudgeConfigOutputExamplesInput']
    output: str
    pass_: bool = Field(alias='pass')
    reasoning: Optional[str]

class GraderDataGraderConfigJudgeConfigOutputExamplesInput(BaseModel):
    """@public"""
    role: str
    content: str

class GraderDataGraderConfigJudgeConfigOutputModel(ModelData):
    """@public"""
    pass

class GraderDataGraderConfigPrebuiltConfigOutput(BaseModel):
    """@public"""
    typename__: Literal['PrebuiltConfigOutput'] = Field(alias='__typename')
    prebuilt_criteria: 'GraderDataGraderConfigPrebuiltConfigOutputPrebuiltCriteria' = Field(alias='prebuiltCriteria')
    model: 'GraderDataGraderConfigPrebuiltConfigOutputModel'

class GraderDataGraderConfigPrebuiltConfigOutputPrebuiltCriteria(BaseModel):
    """@public"""
    key: PrebuiltCriteriaKey
    name: str
    feedback_key: str = Field(alias='feedbackKey')
    description: str

class GraderDataGraderConfigPrebuiltConfigOutputModel(ModelData):
    """@public"""
    pass

class GraderDataGraderConfigRemoteConfigOutput(BaseModel):
    """@public"""
    typename__: Literal['RemoteConfigOutput'] = Field(alias='__typename')
    url: str
    version: str
    description: str

class GraderDataGraderConfigCustomConfigOutput(BaseModel):
    """@public"""
    typename__: Literal['CustomConfigOutput'] = Field(alias='__typename')
    grader_description: Optional[str] = Field(alias='graderDescription')

class GraderDataUseCase(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str

class GraderDataMetric(MetricData):
    """@public"""
    pass

class HarmonyGroupData(BaseModel):
    """@public"""
    id: Any
    key: str
    compute_pool: Optional['HarmonyGroupDataComputePool'] = Field(alias='computePool')
    status: HarmonyStatus
    url: str
    world_size: int = Field(alias='worldSize')
    gpu_types: str = Field(alias='gpuTypes')
    created_at: int = Field(alias='createdAt')
    online_models: List['HarmonyGroupDataOnlineModels'] = Field(alias='onlineModels')

class HarmonyGroupDataComputePool(BaseModel):
    """@public"""
    key: str
    name: str

class HarmonyGroupDataOnlineModels(ModelData):
    """@public"""
    pass

class JobData(BaseModel):
    """@public"""
    id: Any
    name: str
    status: JobStatus
    created_at: int = Field(alias='createdAt')
    created_by: Optional['JobDataCreatedBy'] = Field(alias='createdBy')
    started_at: Optional[int] = Field(alias='startedAt')
    ended_at: Optional[int] = Field(alias='endedAt')
    duration_ms: Optional[int] = Field(alias='durationMs')
    progress: float
    error: Optional[str]
    kind: JobKind
    stages: List['JobDataStages']
    use_case: Optional['JobDataUseCase'] = Field(alias='useCase')
    recipe: Optional['JobDataRecipe']
    details: Optional['JobDataDetails']

class JobDataCreatedBy(BaseModel):
    """@public"""
    id: Any
    name: str

class JobDataStages(BaseModel):
    """@public"""
    name: str
    status: JobStatusOutput
    info: Optional[Annotated[Union['JobDataStagesInfoTrainingJobStageOutput', 'JobDataStagesInfoEvalJobStageOutput', 'JobDataStagesInfoBatchInferenceJobStageOutput'], Field(discriminator='typename__')]]

class JobDataStagesInfoTrainingJobStageOutput(BaseModel):
    """@public"""
    typename__: Literal['TrainingJobStageOutput'] = Field(alias='__typename')
    monitoring_link: Optional[str] = Field(alias='monitoringLink')
    total_num_samples: Optional[int] = Field(alias='totalNumSamples')
    processed_num_samples: Optional[int] = Field(alias='processedNumSamples')
    checkpoints: List[str]

class JobDataStagesInfoEvalJobStageOutput(BaseModel):
    """@public"""
    typename__: Literal['EvalJobStageOutput'] = Field(alias='__typename')
    total_num_samples: Optional[int] = Field(alias='totalNumSamples')
    processed_num_samples: Optional[int] = Field(alias='processedNumSamples')

class JobDataStagesInfoBatchInferenceJobStageOutput(BaseModel):
    """@public"""
    typename__: Literal['BatchInferenceJobStageOutput'] = Field(alias='__typename')
    total_num_samples: Optional[int] = Field(alias='totalNumSamples')
    processed_num_samples: Optional[int] = Field(alias='processedNumSamples')

class JobDataUseCase(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str

class JobDataRecipe(CustomRecipeData):
    """@public"""
    pass

class JobDataDetails(BaseModel):
    """@public"""
    args: Any
    recipe_hash: str = Field(alias='recipeHash')
    artifacts: List['JobDataDetailsArtifacts']

class JobDataDetailsArtifacts(BaseModel):
    """@public"""
    id: Any
    name: str
    kind: JobArtifactKind
    uri: Optional[str]
    metadata: Any
    created_at: int = Field(alias='createdAt')

class JobStageOutputData(BaseModel):
    """@public"""
    name: str
    status: JobStatusOutput
    parent: Optional[str]
    stage_id: int = Field(alias='stageId')
    info: Optional[Annotated[Union['JobStageOutputDataInfoTrainingJobStageOutput', 'JobStageOutputDataInfoEvalJobStageOutput', 'JobStageOutputDataInfoBatchInferenceJobStageOutput'], Field(discriminator='typename__')]]
    started_at: Optional[int] = Field(alias='startedAt')
    ended_at: Optional[int] = Field(alias='endedAt')

class JobStageOutputDataInfoTrainingJobStageOutput(BaseModel):
    """@public"""
    typename__: Literal['TrainingJobStageOutput'] = Field(alias='__typename')
    monitoring_link: Optional[str] = Field(alias='monitoringLink')
    total_num_samples: Optional[int] = Field(alias='totalNumSamples')
    processed_num_samples: Optional[int] = Field(alias='processedNumSamples')
    checkpoints: List[str]

class JobStageOutputDataInfoEvalJobStageOutput(BaseModel):
    """@public"""
    typename__: Literal['EvalJobStageOutput'] = Field(alias='__typename')
    total_num_samples: Optional[int] = Field(alias='totalNumSamples')
    processed_num_samples: Optional[int] = Field(alias='processedNumSamples')
    monitoring_link: Optional[str] = Field(alias='monitoringLink')

class JobStageOutputDataInfoBatchInferenceJobStageOutput(BaseModel):
    """@public"""
    typename__: Literal['BatchInferenceJobStageOutput'] = Field(alias='__typename')
    total_num_samples: Optional[int] = Field(alias='totalNumSamples')
    processed_num_samples: Optional[int] = Field(alias='processedNumSamples')
    monitoring_link: Optional[str] = Field(alias='monitoringLink')

class JudgeData(BaseModel):
    """@public"""
    id: str
    key: str
    version: int
    name: str
    criteria: Optional[str]
    prebuilt: Optional[str]
    examples: Optional[List['JudgeDataExamples']]
    capabilities: List[JudgeCapability]
    model: 'JudgeDataModel'
    use_case_id: Any = Field(alias='useCaseId')
    metric: 'JudgeDataMetric'
    created_at: int = Field(alias='createdAt')
    updated_at: int = Field(alias='updatedAt')

class JudgeDataExamples(BaseModel):
    """@public"""
    input: List['JudgeDataExamplesInput']
    output: str
    pass_: bool = Field(alias='pass')
    reasoning: Optional[str]

class JudgeDataExamplesInput(BaseModel):
    """@public"""
    role: str
    content: str

class JudgeDataModel(ModelData):
    """@public"""
    pass

class JudgeDataMetric(MetricData):
    """@public"""
    pass

class MetricDataAdmin(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str
    kind: MetricKind
    description: str
    scoring_type: MetricScoringType = Field(alias='scoringType')
    use_cases: List['MetricDataAdminUseCases'] = Field(alias='useCases')
    created_at: int = Field(alias='createdAt')
    has_direct_feedbacks: bool = Field(alias='hasDirectFeedbacks')
    has_comparison_feedbacks: bool = Field(alias='hasComparisonFeedbacks')

class MetricDataAdminUseCases(BaseModel):
    """@public"""
    id: Any
    name: str
    key: str
    description: str

class MetricWithContextData(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str
    kind: MetricKind
    description: str
    scoring_type: MetricScoringType = Field(alias='scoringType')
    created_at: Any = Field(alias='createdAt')

class ModelDataAdmin(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str
    online: ModelOnline
    use_cases: List['ModelDataAdminUseCases'] = Field(alias='useCases')
    is_external: bool = Field(alias='isExternal')
    provider_name: ProviderName = Field(alias='providerName')
    is_adapter: bool = Field(alias='isAdapter')
    is_training: bool = Field(alias='isTraining')
    created_at: int = Field(alias='createdAt')
    kind: ModelKindFilter
    size: Optional[str]

class ModelDataAdminUseCases(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str

class ModelServiceData(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str
    model: 'ModelServiceDataModel'
    attached: bool
    is_default: bool = Field(alias='isDefault')
    desired_online: bool = Field(alias='desiredOnline')
    created_at: int = Field(alias='createdAt')

class ModelServiceDataModel(ModelData):
    """@public"""
    backbone: Optional['ModelServiceDataModelBackbone']

class ModelServiceDataModelBackbone(ModelData):
    """@public"""
    pass

class RemoteEnvData(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str
    url: str
    description: str
    created_at: int = Field(alias='createdAt')
    version: str
    status: RemoteEnvStatus
    metadata_schema: Optional[Any] = Field(alias='metadataSchema')

class UseCaseData(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str
    description: str
    created_at: int = Field(alias='createdAt')
    metrics: List['UseCaseDataMetrics']
    model_services: List['UseCaseDataModelServices'] = Field(alias='modelServices')
    permissions: List[str]
    shares: List['UseCaseDataShares']

class UseCaseDataMetrics(MetricWithContextData):
    """@public"""
    pass

class UseCaseDataModelServices(ModelServiceData):
    """@public"""
    pass

class UseCaseDataShares(BaseModel):
    """@public"""
    team: 'UseCaseDataSharesTeam'
    role: 'UseCaseDataSharesRole'
    is_owner: bool = Field(alias='isOwner')

class UseCaseDataSharesTeam(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str
    created_at: int = Field(alias='createdAt')

class UseCaseDataSharesRole(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str
    created_at: int = Field(alias='createdAt')
    permissions: List[str]

class UserData(BaseModel):
    """@public"""
    id: Any
    email: str
    name: str
    created_at: int = Field(alias='createdAt')
    teams: List['UserDataTeams']

class UserDataTeams(BaseModel):
    """@public"""
    team: 'UserDataTeamsTeam'
    role: 'UserDataTeamsRole'

class UserDataTeamsTeam(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str
    created_at: int = Field(alias='createdAt')

class UserDataTeamsRole(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str
    created_at: int = Field(alias='createdAt')
    permissions: List[str]
AbCampaignCreateData.model_rebuild()
MetricData.model_rebuild()
AbCampaignDetailData.model_rebuild()
AbCampaignReportData.model_rebuild()
CompletionComparisonFeedbackData.model_rebuild()
CompletionData.model_rebuild()
CustomRecipeData.model_rebuild()
DatasetData.model_rebuild()
ModelData.model_rebuild()
GraderData.model_rebuild()
HarmonyGroupData.model_rebuild()
JobData.model_rebuild()
JobStageOutputData.model_rebuild()
JudgeData.model_rebuild()
MetricDataAdmin.model_rebuild()
MetricWithContextData.model_rebuild()
ModelDataAdmin.model_rebuild()
ModelServiceData.model_rebuild()
RemoteEnvData.model_rebuild()
UseCaseData.model_rebuild()
UserData.model_rebuild()


from typing import Any, Dict, Optional
from . import CompletionGroupBy
from .custom_fields import AbcampaignFields, CompletionConnectionFields, CompletionFields, CompletionGroupDataConnectionFields, ComputePoolFields, ContractFields, CustomRecipeFields, DatasetFields, GlobalUsageFields, GraderFields, HarmonyGroupFields, JobArtifactFields, JobConnectionFields, JobFields, JudgeFields, MetaObjectFields, MetricFields, ModelFields, PrebuiltConfigDefinitionFields, PrebuiltCriteriaFields, RemoteConfigOutputFields, RemoteEnvFields, RoleFields, SystemPromptTemplateFields, TeamFields, ToolProviderFields, UsageAggregateItemFields, UsageAggregatePerUseCaseItemFields, UseCaseFields, UserFields
from .custom_typing_fields import GraphQLField
from .input_types import AbCampaignFilter, ArtifactFilter, CursorPageInput, CustomRecipeFilterInput, DatasetCreateFromFilters, FeedbackFilterInput, GlobalUsageFilterInput, ListCompletionsFilterInput, ListJobsFilterInput, ModelFilter, OrderPair, UsageFilterInput, UsagePerUseCaseFilterInput, UseCaseFilter

class Query:
    """@private"""

    @classmethod
    def ab_campaigns(cls, filter: AbCampaignFilter) -> AbcampaignFields:
        arguments: Dict[str, Dict[str, Any]] = {'filter': {'type': 'AbCampaignFilter!', 'value': filter}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return AbcampaignFields(field_name='abCampaigns', arguments=cleared_arguments)

    @classmethod
    def ab_campaign(cls, id_or_key: str) -> AbcampaignFields:
        arguments: Dict[str, Dict[str, Any]] = {'idOrKey': {'type': 'IdOrKey!', 'value': id_or_key}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return AbcampaignFields(field_name='abCampaign', arguments=cleared_arguments)

    @classmethod
    def contract(cls) -> ContractFields:
        return ContractFields(field_name='contract')

    @classmethod
    def custom_recipes(cls, use_case: str, filter: CustomRecipeFilterInput) -> CustomRecipeFields:
        arguments: Dict[str, Dict[str, Any]] = {'useCase': {'type': 'IdOrKey!', 'value': use_case}, 'filter': {'type': 'CustomRecipeFilterInput!', 'value': filter}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return CustomRecipeFields(field_name='customRecipes', arguments=cleared_arguments)

    @classmethod
    def custom_recipe(cls, id_or_key: str, use_case: str) -> CustomRecipeFields:
        arguments: Dict[str, Dict[str, Any]] = {'idOrKey': {'type': 'IdOrKey!', 'value': id_or_key}, 'useCase': {'type': 'IdOrKey!', 'value': use_case}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return CustomRecipeFields(field_name='customRecipe', arguments=cleared_arguments)

    @classmethod
    def datasets(cls, use_case: str) -> DatasetFields:
        arguments: Dict[str, Dict[str, Any]] = {'useCase': {'type': 'IdOrKey!', 'value': use_case}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return DatasetFields(field_name='datasets', arguments=cleared_arguments)

    @classmethod
    def dataset(cls, id_or_key: str, use_case: str) -> DatasetFields:
        arguments: Dict[str, Dict[str, Any]] = {'idOrKey': {'type': 'IdOrKey!', 'value': id_or_key}, 'useCase': {'type': 'IdOrKey!', 'value': use_case}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return DatasetFields(field_name='dataset', arguments=cleared_arguments)

    @classmethod
    def preview_dataset_from_filters(cls, input: DatasetCreateFromFilters) -> GraphQLField:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'DatasetCreateFromFilters!', 'value': input}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return GraphQLField(field_name='previewDatasetFromFilters', arguments=cleared_arguments)

    @classmethod
    def completions(cls, filter: ListCompletionsFilterInput, page: CursorPageInput, order: OrderPair) -> CompletionConnectionFields:
        arguments: Dict[str, Dict[str, Any]] = {'filter': {'type': 'ListCompletionsFilterInput!', 'value': filter}, 'page': {'type': 'CursorPageInput!', 'value': page}, 'order': {'type': 'OrderPair!', 'value': order}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return CompletionConnectionFields(field_name='completions', arguments=cleared_arguments)

    @classmethod
    def completions_grouped(cls, filter: ListCompletionsFilterInput, feedback_filter: FeedbackFilterInput, group_by: CompletionGroupBy, page: CursorPageInput, order: OrderPair) -> CompletionGroupDataConnectionFields:
        arguments: Dict[str, Dict[str, Any]] = {'filter': {'type': 'ListCompletionsFilterInput!', 'value': filter}, 'feedbackFilter': {'type': 'FeedbackFilterInput!', 'value': feedback_filter}, 'groupBy': {'type': 'CompletionGroupBy!', 'value': group_by}, 'page': {'type': 'CursorPageInput!', 'value': page}, 'order': {'type': 'OrderPair!', 'value': order}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return CompletionGroupDataConnectionFields(field_name='completionsGrouped', arguments=cleared_arguments)

    @classmethod
    def completion(cls, use_case: str, id: Any) -> CompletionFields:
        arguments: Dict[str, Dict[str, Any]] = {'useCase': {'type': 'IdOrKey!', 'value': use_case}, 'id': {'type': 'UUID!', 'value': id}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return CompletionFields(field_name='completion', arguments=cleared_arguments)

    @classmethod
    def completion_download_url(cls, filter: ListCompletionsFilterInput) -> GraphQLField:
        arguments: Dict[str, Dict[str, Any]] = {'filter': {'type': 'ListCompletionsFilterInput!', 'value': filter}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return GraphQLField(field_name='completionDownloadUrl', arguments=cleared_arguments)

    @classmethod
    def model_usage(cls, filter: UsageFilterInput) -> UsageAggregateItemFields:
        arguments: Dict[str, Dict[str, Any]] = {'filter': {'type': 'UsageFilterInput!', 'value': filter}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return UsageAggregateItemFields(field_name='modelUsage', arguments=cleared_arguments)

    @classmethod
    def model_usage_by_use_case(cls, filter: UsagePerUseCaseFilterInput) -> UsageAggregatePerUseCaseItemFields:
        arguments: Dict[str, Dict[str, Any]] = {'filter': {'type': 'UsagePerUseCaseFilterInput!', 'value': filter}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return UsageAggregatePerUseCaseItemFields(field_name='modelUsageByUseCase', arguments=cleared_arguments)

    @classmethod
    def global_usage(cls, filter: GlobalUsageFilterInput) -> GlobalUsageFields:
        arguments: Dict[str, Dict[str, Any]] = {'filter': {'type': 'GlobalUsageFilterInput!', 'value': filter}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return GlobalUsageFields(field_name='globalUsage', arguments=cleared_arguments)

    @classmethod
    def system_prompt_templates(cls) -> SystemPromptTemplateFields:
        return SystemPromptTemplateFields(field_name='systemPromptTemplates')

    @classmethod
    def jobs(cls, page: CursorPageInput, filter: ListJobsFilterInput, order: OrderPair) -> JobConnectionFields:
        arguments: Dict[str, Dict[str, Any]] = {'page': {'type': 'CursorPageInput!', 'value': page}, 'filter': {'type': 'ListJobsFilterInput!', 'value': filter}, 'order': {'type': 'OrderPair!', 'value': order}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return JobConnectionFields(field_name='jobs', arguments=cleared_arguments)

    @classmethod
    def job(cls, id: Any) -> JobFields:
        arguments: Dict[str, Dict[str, Any]] = {'id': {'type': 'UUID!', 'value': id}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return JobFields(field_name='job', arguments=cleared_arguments)

    @classmethod
    def metrics(cls) -> MetricFields:
        return MetricFields(field_name='metrics')

    @classmethod
    def metric(cls, id_or_key: str) -> MetricFields:
        arguments: Dict[str, Dict[str, Any]] = {'idOrKey': {'type': 'IdOrKey!', 'value': id_or_key}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return MetricFields(field_name='metric', arguments=cleared_arguments)

    @classmethod
    def models(cls, filter: ModelFilter) -> ModelFields:
        arguments: Dict[str, Dict[str, Any]] = {'filter': {'type': 'ModelFilter!', 'value': filter}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return ModelFields(field_name='models', arguments=cleared_arguments)

    @classmethod
    def model(cls, id_or_key: str) -> ModelFields:
        arguments: Dict[str, Dict[str, Any]] = {'idOrKey': {'type': 'IdOrKey!', 'value': id_or_key}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return ModelFields(field_name='model', arguments=cleared_arguments)

    @classmethod
    def all_harmony_groups(cls) -> HarmonyGroupFields:
        return HarmonyGroupFields(field_name='allHarmonyGroups')

    @classmethod
    def harmony_groups(cls) -> HarmonyGroupFields:
        return HarmonyGroupFields(field_name='harmonyGroups')

    @classmethod
    def compute_pools(cls) -> ComputePoolFields:
        return ComputePoolFields(field_name='computePools')

    @classmethod
    def remote_envs(cls) -> RemoteEnvFields:
        return RemoteEnvFields(field_name='remoteEnvs')

    @classmethod
    def use_cases(cls, filter: UseCaseFilter) -> UseCaseFields:
        arguments: Dict[str, Dict[str, Any]] = {'filter': {'type': 'UseCaseFilter!', 'value': filter}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return UseCaseFields(field_name='useCases', arguments=cleared_arguments)

    @classmethod
    def use_case(cls, id_or_key: str) -> UseCaseFields:
        arguments: Dict[str, Dict[str, Any]] = {'idOrKey': {'type': 'IdOrKey!', 'value': id_or_key}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return UseCaseFields(field_name='useCase', arguments=cleared_arguments)

    @classmethod
    def me(cls) -> UserFields:
        return UserFields(field_name='me')

    @classmethod
    def users(cls) -> UserFields:
        return UserFields(field_name='users')

    @classmethod
    def roles(cls) -> RoleFields:
        return RoleFields(field_name='roles')

    @classmethod
    def permissions(cls) -> GraphQLField:
        return GraphQLField(field_name='permissions')

    @classmethod
    def teams(cls) -> TeamFields:
        return TeamFields(field_name='teams')

    @classmethod
    def judge(cls, id: str, use_case: str, *, version: Optional[int]=None) -> JudgeFields:
        arguments: Dict[str, Dict[str, Any]] = {'id': {'type': 'IdOrKey!', 'value': id}, 'useCase': {'type': 'IdOrKey!', 'value': use_case}, 'version': {'type': 'Int', 'value': version}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return JudgeFields(field_name='judge', arguments=cleared_arguments)

    @classmethod
    def judges(cls, use_case: str) -> JudgeFields:
        arguments: Dict[str, Dict[str, Any]] = {'useCase': {'type': 'IdOrKey!', 'value': use_case}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return JudgeFields(field_name='judges', arguments=cleared_arguments)

    @classmethod
    def judge_versions(cls, use_case: str, key: str) -> JudgeFields:
        arguments: Dict[str, Dict[str, Any]] = {'useCase': {'type': 'IdOrKey!', 'value': use_case}, 'key': {'type': 'String!', 'value': key}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return JudgeFields(field_name='judgeVersions', arguments=cleared_arguments)

    @classmethod
    def prebuilt_criteria(cls) -> PrebuiltCriteriaFields:
        return PrebuiltCriteriaFields(field_name='prebuiltCriteria')

    @classmethod
    def grader(cls, id: str, use_case: str) -> GraderFields:
        arguments: Dict[str, Dict[str, Any]] = {'id': {'type': 'IdOrKey!', 'value': id}, 'useCase': {'type': 'IdOrKey!', 'value': use_case}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return GraderFields(field_name='grader', arguments=cleared_arguments)

    @classmethod
    def graders(cls, use_case: str) -> GraderFields:
        arguments: Dict[str, Dict[str, Any]] = {'useCase': {'type': 'IdOrKey!', 'value': use_case}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return GraderFields(field_name='graders', arguments=cleared_arguments)

    @classmethod
    def prebuilt_configs(cls) -> PrebuiltConfigDefinitionFields:
        return PrebuiltConfigDefinitionFields(field_name='prebuiltConfigs')

    @classmethod
    def test_remote_env_2(cls, url: str) -> RemoteConfigOutputFields:
        arguments: Dict[str, Dict[str, Any]] = {'url': {'type': 'String!', 'value': url}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return RemoteConfigOutputFields(field_name='testRemoteEnv2', arguments=cleared_arguments)

    @classmethod
    def validate_data_schema_for_grader(cls, grader: str, dataset: str, usecase: str) -> GraphQLField:
        arguments: Dict[str, Dict[str, Any]] = {'grader': {'type': 'IdOrKey!', 'value': grader}, 'dataset': {'type': 'IdOrKey!', 'value': dataset}, 'usecase': {'type': 'IdOrKey!', 'value': usecase}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return GraphQLField(field_name='validateDataSchemaForGrader', arguments=cleared_arguments)

    @classmethod
    def tool_provider(cls, id_or_key: str, use_case: str) -> ToolProviderFields:
        arguments: Dict[str, Dict[str, Any]] = {'idOrKey': {'type': 'IdOrKey!', 'value': id_or_key}, 'useCase': {'type': 'IdOrKey!', 'value': use_case}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return ToolProviderFields(field_name='toolProvider', arguments=cleared_arguments)

    @classmethod
    def artifacts(cls, use_case: str, *, filter: Optional[ArtifactFilter]=None) -> JobArtifactFields:
        arguments: Dict[str, Dict[str, Any]] = {'useCase': {'type': 'IdOrKey!', 'value': use_case}, 'filter': {'type': 'ArtifactFilter', 'value': filter}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return JobArtifactFields(field_name='artifacts', arguments=cleared_arguments)

    @classmethod
    def artifact(cls, use_case: str, id: Any) -> JobArtifactFields:
        arguments: Dict[str, Dict[str, Any]] = {'useCase': {'type': 'IdOrKey!', 'value': use_case}, 'id': {'type': 'UUID!', 'value': id}}
        cleared_arguments = {key: value for (key, value) in arguments.items() if value['value'] is not None}
        return JobArtifactFields(field_name='artifact', arguments=cleared_arguments)

    @classmethod
    def meta(cls) -> MetaObjectFields:
        return MetaObjectFields(field_name='meta')


from typing import List, Optional
from pydantic import Field
from .base_model import BaseModel
from .fragments import CompletionData

class ListInteractions(BaseModel):
    """@public"""
    completions: 'ListInteractionsCompletions'

class ListInteractionsCompletions(BaseModel):
    """@public"""
    total_count: int = Field(alias='totalCount')
    page_info: 'ListInteractionsCompletionsPageInfo' = Field(alias='pageInfo')
    nodes: List['ListInteractionsCompletionsNodes']

class ListInteractionsCompletionsPageInfo(BaseModel):
    """@public"""
    has_next_page: bool = Field(alias='hasNextPage')
    end_cursor: Optional[str] = Field(alias='endCursor')

class ListInteractionsCompletionsNodes(CompletionData):
    """@public"""
    pass
ListInteractions.model_rebuild()
ListInteractionsCompletions.model_rebuild()


from pydantic import Field
from .base_model import BaseModel
from .fragments import MetricData

class CreateMetric(BaseModel):
    """@public"""
    create_metric: 'CreateMetricCreateMetric' = Field(alias='createMetric')

class CreateMetricCreateMetric(MetricData):
    """@public"""
    pass
CreateMetric.model_rebuild()


from pydantic import Field
from .base_model import BaseModel
from .fragments import GraderData

class CreateGrader(BaseModel):
    """@public"""
    create_grader: 'CreateGraderCreateGrader' = Field(alias='createGrader')

class CreateGraderCreateGrader(GraderData):
    """@public"""
    pass
CreateGrader.model_rebuild()


from typing import Optional
from .base_model import BaseModel
from .fragments import CompletionData

class DescribeInteraction(BaseModel):
    """@public"""
    completion: Optional['DescribeInteractionCompletion']

class DescribeInteractionCompletion(CompletionData):
    """@public"""
    pass
DescribeInteraction.model_rebuild()